@@ -63,6 +63,7 @@
 #include "base/logger.h"
 #include "base/preferences.h"
 #include "base/settingsstorage.h"
+#include "base/profile.h"
 #include "base/utils/fs.h"
 #include "base/utils/misc.h"
 #include "base/iconprovider.h"
@@ -70,8 +71,11 @@
 #include "base/net/smtp.h"
 #include "base/net/downloadmanager.h"
 #include "base/net/geoipmanager.h"
+#include "base/net/proxyconfigurationmanager.h"
 #include "base/bittorrent/session.h"
 #include "base/bittorrent/torrenthandle.h"
+#include "base/rss/rss_autodownloader.h"
+#include "base/rss/rss_session.h"
 
 namespace
 {
@@ -92,35 +96,49 @@ namespace
 
     const QString LOG_FOLDER("logs");
     const char PARAMS_SEPARATOR[] = "|";
+
+    const QString DEFAULT_PORTABLE_MODE_PROFILE_DIR = QLatin1String("profile");
 }
 
 Application::Application(const QString &id, int &argc, char **argv)
     : BaseApplication(id, argc, argv)
     , m_running(false)
     , m_shutdownAct(ShutdownDialogAction::Exit)
+    , m_commandLineArgs(parseCommandLine(this->arguments()))
+#ifndef DISABLE_WEBUI
+    , m_webui(nullptr)
+#endif
 {
+    qRegisterMetaType<Log::Msg>("Log::Msg");
+
+    setApplicationName("qBittorrent");
+    validateCommandLineParameters();
+
+    QString profileDir = m_commandLineArgs.portableMode
+        ? QDir(QCoreApplication::applicationDirPath()).absoluteFilePath(DEFAULT_PORTABLE_MODE_PROFILE_DIR)
+        : m_commandLineArgs.profileDir;
+
+    Profile::initialize(profileDir, m_commandLineArgs.configurationName,
+                        m_commandLineArgs.relativeFastresumePaths || m_commandLineArgs.portableMode);
+
     Logger::initInstance();
     SettingsStorage::initInstance();
     Preferences::initInstance();
 
-#if defined(Q_OS_MACX) && !defined(DISABLE_GUI)
-    if (QSysInfo::MacintoshVersion > QSysInfo::MV_10_8) {
-        // fix Mac OS X 10.9 (mavericks) font issue
-        // https://bugreports.qt-project.org/browse/QTBUG-32789
-        QFont::insertSubstitution(".Lucida Grande UI", "Lucida Grande");
-    }
-#endif
+    if (m_commandLineArgs.webUiPort > 0) // it will be -1 when user did not set any value
+        Preferences::instance()->setWebUiPort(m_commandLineArgs.webUiPort);
+
     setApplicationName("qBittorrent");
     initializeTranslation();
-#ifndef DISABLE_GUI
-#ifdef QBT_USES_QT5
+
+#if !defined(DISABLE_GUI)
     setAttribute(Qt::AA_UseHighDpiPixmaps, true);  // opt-in to the high DPI pixmap support
-#endif // QBT_USES_QT5
     setQuitOnLastWindowClosed(false);
-#ifdef Q_OS_WIN
+#endif
+
+#if defined(Q_OS_WIN) && !defined(DISABLE_GUI)
     connect(this, SIGNAL(commitDataRequest(QSessionManager &)), this, SLOT(shutdownCleanup(QSessionManager &)), Qt::DirectConnection);
-#endif // Q_OS_WIN
-#endif // DISABLE_GUI
+#endif
 
     connect(this, SIGNAL(messageReceived(const QString &)), SLOT(processMessage(const QString &)));
     connect(this, SIGNAL(aboutToQuit()), SLOT(cleanup()));
@@ -128,7 +146,14 @@ Application::Application(const QString &id, int &argc, char **argv)
     if (isFileLoggerEnabled())
         m_fileLogger = new FileLogger(fileLoggerPath(), isFileLoggerBackup(), fileLoggerMaxSize(), isFileLoggerDeleteOld(), fileLoggerAge(), static_cast<FileLogger::FileLogAgeType>(fileLoggerAgeType()));
 
-    Logger::instance()->addMessage(tr("qBittorrent %1 started", "qBittorrent v3.2.0alpha started").arg(VERSION));
+    Logger::instance()->addMessage(tr("qBittorrent %1 started", "qBittorrent v3.2.0alpha started").arg(QBT_VERSION));
+}
+
+Application::~Application()
+{
+    // we still need to call cleanup()
+    // in case the App failed to start
+    cleanup();
 }
 
 #ifndef DISABLE_GUI
@@ -138,6 +163,11 @@ QPointer<MainWindow> Application::mainWindow()
 }
 #endif
 
+const QBtCommandLineParameters &Application::commandLineArgs() const
+{
+    return m_commandLineArgs;
+}
+
 bool Application::isFileLoggerEnabled() const
 {
     return settings()->loadValue(KEY_FILELOGGER_ENABLED, true).toBool();
@@ -154,14 +184,15 @@ void Application::setFileLoggerEnabled(bool value)
 
 QString Application::fileLoggerPath() const
 {
-    return settings()->loadValue(KEY_FILELOGGER_PATH, QVariant(Utils::Fs::QDesktopServicesDataLocation() + LOG_FOLDER)).toString();
+    return settings()->loadValue(KEY_FILELOGGER_PATH,
+            QVariant(specialFolderLocation(SpecialFolder::Data) + LOG_FOLDER)).toString();
 }
 
-void Application::setFileLoggerPath(const QString &value)
+void Application::setFileLoggerPath(const QString &path)
 {
     if (m_fileLogger)
-        m_fileLogger->changePath(value);
-    settings()->storeValue(KEY_FILELOGGER_PATH, value);
+        m_fileLogger->changePath(path);
+    settings()->storeValue(KEY_FILELOGGER_PATH, path);
 }
 
 bool Application::isFileLoggerBackup() const
@@ -263,7 +294,7 @@ void Application::runExternalProgram(BitTorrent::TorrentHandle *const torrent) c
 #elif defined(Q_OS_WIN)  // test cmd: `echo "%F" > "c:\ab ba.txt"`
     program.prepend(QLatin1String("\"")).append(QLatin1String("\""));
     program.prepend(Utils::Misc::windowsSystemPath() + QLatin1String("\\cmd.exe /C "));
-    const uint cmdMaxLength = 32768;  // max length (incl. terminate char) for `lpCommandLine` in `CreateProcessW()`
+    const int cmdMaxLength = 32768;  // max length (incl. terminate char) for `lpCommandLine` in `CreateProcessW()`
     if ((program.size() + 1) > cmdMaxLength) {
         logger->addMessage(tr("Torrent: %1, run external program command too long (length > %2), execution failed.").arg(torrent->name()).arg(cmdMaxLength), Log::CRITICAL);
         return;
@@ -286,22 +317,22 @@ void Application::runExternalProgram(BitTorrent::TorrentHandle *const torrent) c
 #endif
 }
 
-void Application::sendNotificationEmail(BitTorrent::TorrentHandle *const torrent)
+void Application::sendNotificationEmail(const BitTorrent::TorrentHandle *torrent)
 {
     // Prepare mail content
-    QString content = QObject::tr("Torrent name: %1").arg(torrent->name()) + "\n";
-    content += QObject::tr("Torrent size: %1").arg(Utils::Misc::friendlyUnit(torrent->wantedSize())) + "\n";
-    content += QObject::tr("Save path: %1").arg(torrent->savePath()) + "\n\n";
-    content += QObject::tr("The torrent was downloaded in %1.",
-                  "The torrent was downloaded in 1 hour and 20 seconds")
-            .arg(Utils::Misc::userFriendlyDuration(torrent->activeTime())) + "\n\n\n";
-    content += QObject::tr("Thank you for using qBittorrent.") + "\n";
+    const QString content = tr("Torrent name: %1").arg(torrent->name()) + "\n"
+        + tr("Torrent size: %1").arg(Utils::Misc::friendlyUnit(torrent->wantedSize())) + "\n"
+        + tr("Save path: %1").arg(torrent->savePath()) + "\n\n"
+        + tr("The torrent was downloaded in %1.", "The torrent was downloaded in 1 hour and 20 seconds")
+            .arg(Utils::Misc::userFriendlyDuration(torrent->activeTime())) + "\n\n\n"
+        + tr("Thank you for using qBittorrent.") + "\n";
 
     // Send the notification email
-    Net::Smtp *sender = new Net::Smtp;
-    sender->sendMail("notification@qbittorrent.org",
-                     Preferences::instance()->getMailNotificationEmail(),
-                     QObject::tr("[qBittorrent] '%1' has finished downloading").arg(torrent->name()),
+    const Preferences *pref = Preferences::instance();
+    Net::Smtp *smtp = new Net::Smtp(this);
+    smtp->sendMail(pref->getMailNotificationSender(),
+                     pref->getMailNotificationEmail(),
+                     tr("[qBittorrent] '%1' has finished downloading").arg(torrent->name()),
                      content);
 }
 
@@ -345,7 +376,7 @@ void Application::allTorrentsFinished()
         // do nothing & skip confirm
     }
     else {
-        if (!ShutdownConfirmDlg::askForConfirmation(action)) return;
+        if (!ShutdownConfirmDlg::askForConfirmation(m_window, action)) return;
     }
 #endif // DISABLE_GUI
 
@@ -381,20 +412,69 @@ void Application::processParams(const QStringList &params)
         return;
     }
 #endif
+    BitTorrent::AddTorrentParams torrentParams;
+    TriStateBool skipTorrentDialog;
 
     foreach (QString param, params) {
         param = param.trimmed();
+
+        // Process strings indicating options specified by the user.
+
+        if (param.startsWith(QLatin1String("@savePath="))) {
+            torrentParams.savePath = param.mid(10);
+            continue;
+        }
+
+        if (param.startsWith(QLatin1String("@addPaused="))) {
+            torrentParams.addPaused = param.mid(11).toInt() ? TriStateBool::True : TriStateBool::False;
+            continue;
+        }
+
+        if (param == QLatin1String("@skipChecking")) {
+            torrentParams.skipChecking = true;
+            continue;
+        }
+
+        if (param.startsWith(QLatin1String("@category="))) {
+            torrentParams.category = param.mid(10);
+            continue;
+        }
+
+        if (param == QLatin1String("@sequential")) {
+            torrentParams.sequential = true;
+            continue;
+        }
+
+        if (param == QLatin1String("@firstLastPiecePriority")) {
+            torrentParams.firstLastPiecePriority = true;
+            continue;
+        }
+
+        if (param.startsWith(QLatin1String("@skipDialog="))) {
+            skipTorrentDialog = param.mid(12).toInt() ? TriStateBool::True : TriStateBool::False;
+            continue;
+        }
+
 #ifndef DISABLE_GUI
-        if (AddNewTorrentDialog::isEnabled())
-            AddNewTorrentDialog::show(param, m_window);
+        // There are two circumstances in which we want to show the torrent
+        // dialog. One is when the application settings specify that it should
+        // be shown and skipTorrentDialog is undefined. The other is when
+        // skipTorrentDialog is false, meaning that the application setting
+        // should be overridden.
+        const bool showDialogForThisTorrent =
+            ((AddNewTorrentDialog::isEnabled() && skipTorrentDialog == TriStateBool::Undefined)
+             || skipTorrentDialog == TriStateBool::False);
+        if (showDialogForThisTorrent)
+            AddNewTorrentDialog::show(param, torrentParams, m_window);
         else
 #endif
-            BitTorrent::Session::instance()->addTorrent(param);
+            BitTorrent::Session::instance()->addTorrent(param, torrentParams);
     }
 }
 
 int Application::exec(const QStringList &params)
 {
+    Net::ProxyConfigurationManager::initInstance();
     Net::DownloadManager::initInstance();
 #ifdef DISABLE_GUI
     IconProvider::initInstance();
@@ -413,7 +493,15 @@ int Application::exec(const QStringList &params)
 
 #ifndef DISABLE_WEBUI
     m_webui = new WebUI;
-#endif
+#ifdef DISABLE_GUI
+    if (m_webui->isErrored())
+        return 1;
+    connect(m_webui, &WebUI::fatalError, this, []() { QCoreApplication::exit(1); });
+#endif // DISABLE_GUI
+#endif // DISABLE_WEBUI
+
+    new RSS::Session; // create RSS::Session singleton
+    new RSS::AutoDownloader; // create RSS::AutoDownloader singleton
 
 #ifdef DISABLE_GUI
 #ifndef DISABLE_WEBUI
@@ -433,12 +521,15 @@ int Application::exec(const QStringList &params)
 #endif // DISABLE_GUI
 
     m_running = true;
+
+    // Now UI is ready to process signals from Session
+    BitTorrent::Session::instance()->startUpTorrents();
+
     m_paramsQueue = params + m_paramsQueue;
     if (!m_paramsQueue.isEmpty()) {
         processParams(m_paramsQueue);
         m_paramsQueue.clear();
     }
-
     return BaseApplication::exec();
 }
 
@@ -478,7 +569,7 @@ bool Application::event(QEvent *ev)
         if (path.isEmpty())
             // Get the url instead
             path = static_cast<QFileOpenEvent *>(ev)->url().toString();
-        qDebug("Received a mac file open event: %s", qPrintable(path));
+        qDebug("Received a mac file open event: %s", qUtf8Printable(path));
         if (m_running)
             processParams(QStringList(path));
         else
@@ -509,36 +600,24 @@ void Application::initializeTranslation()
 {
     Preferences* const pref = Preferences::instance();
     // Load translation
-    QString locale = pref->getLocale();
+    QString localeStr = pref->getLocale();
 
-    if (locale.isEmpty()) {
-        locale = QLocale::system().name();
-        pref->setLocale(locale);
-    }
+    if (m_qtTranslator.load(QLatin1String("qtbase_") + localeStr, QLibraryInfo::location(QLibraryInfo::TranslationsPath)) ||
+        m_qtTranslator.load(QLatin1String("qt_") + localeStr, QLibraryInfo::location(QLibraryInfo::TranslationsPath)))
+            qDebug("Qt %s locale recognized, using translation.", qUtf8Printable(localeStr));
+    else
+        qDebug("Qt %s locale unrecognized, using default (en).", qUtf8Printable(localeStr));
 
-    if (m_qtTranslator.load(
-#ifdef QBT_USES_QT5
-            QString::fromUtf8("qtbase_") + locale, QLibraryInfo::location(QLibraryInfo::TranslationsPath)) ||
-        m_qtTranslator.load(
-#endif
-            QString::fromUtf8("qt_") + locale, QLibraryInfo::location(QLibraryInfo::TranslationsPath))) {
-            qDebug("Qt %s locale recognized, using translation.", qPrintable(locale));
-    }
-    else {
-        qDebug("Qt %s locale unrecognized, using default (en).", qPrintable(locale));
-    }
     installTranslator(&m_qtTranslator);
 
-    if (m_translator.load(QString::fromUtf8(":/lang/qbittorrent_") + locale)) {
-        qDebug("%s locale recognized, using translation.", qPrintable(locale));
-    }
-    else {
-        qDebug("%s locale unrecognized, using default (en).", qPrintable(locale));
-    }
+    if (m_translator.load(QLatin1String(":/lang/qbittorrent_") + localeStr))
+        qDebug("%s locale recognized, using translation.", qUtf8Printable(localeStr));
+    else
+        qDebug("%s locale unrecognized, using default (en).", qUtf8Printable(localeStr));
     installTranslator(&m_translator);
 
 #ifndef DISABLE_GUI
-    if (locale.startsWith("ar") || locale.startsWith("he")) {
+    if (localeStr.startsWith("ar") || localeStr.startsWith("he")) {
         qDebug("Right to Left mode");
         setLayoutDirection(Qt::RightToLeft);
     }
@@ -578,64 +657,70 @@ void Application::shutdownCleanup(QSessionManager &manager)
 
 void Application::cleanup()
 {
-#ifndef DISABLE_GUI
-#ifdef Q_OS_WIN
     // cleanup() can be called multiple times during shutdown. We only need it once.
     static QAtomicInt alreadyDone;
     if (!alreadyDone.testAndSetAcquire(0, 1))
         return;
-#endif // Q_OS_WIN
 
-    // Hide the window and not leave it on screen as
-    // unresponsive. Also for Windows take the WinId
-    // after it's hidden, because hide() may cause a
-    // WinId change.
-    m_window->hide();
+#ifndef DISABLE_GUI
+    if (m_window) {
+        // Hide the window and not leave it on screen as
+        // unresponsive. Also for Windows take the WinId
+        // after it's hidden, because hide() may cause a
+        // WinId change.
+        m_window->hide();
 
 #ifdef Q_OS_WIN
-    typedef BOOL (WINAPI *PSHUTDOWNBRCREATE)(HWND, LPCWSTR);
-    PSHUTDOWNBRCREATE shutdownBRCreate = (PSHUTDOWNBRCREATE)::GetProcAddress(::GetModuleHandleW(L"User32.dll"), "ShutdownBlockReasonCreate");
-    // Only available on Vista+
-    if (shutdownBRCreate)
-        shutdownBRCreate((HWND)m_window->effectiveWinId(), tr("Saving torrent progress...").toStdWString().c_str());
+        typedef BOOL (WINAPI *PSHUTDOWNBRCREATE)(HWND, LPCWSTR);
+        PSHUTDOWNBRCREATE shutdownBRCreate = (PSHUTDOWNBRCREATE)::GetProcAddress(::GetModuleHandleW(L"User32.dll"), "ShutdownBlockReasonCreate");
+        // Only available on Vista+
+        if (shutdownBRCreate)
+            shutdownBRCreate((HWND)m_window->effectiveWinId(), tr("Saving torrent progress...").toStdWString().c_str());
 #endif // Q_OS_WIN
 
-    // Do manual cleanup in MainWindow to force widgets
-    // to save their Preferences, stop all timers and
-    // delete as many widgets as possible to leave only
-    // a 'shell' MainWindow.
-    // We need a valid window handle for Windows Vista+
-    // otherwise the system shutdown will continue even
-    // though we created a ShutdownBlockReason
-    m_window->cleanup();
-
+        // Do manual cleanup in MainWindow to force widgets
+        // to save their Preferences, stop all timers and
+        // delete as many widgets as possible to leave only
+        // a 'shell' MainWindow.
+        // We need a valid window handle for Windows Vista+
+        // otherwise the system shutdown will continue even
+        // though we created a ShutdownBlockReason
+        m_window->cleanup();
+    }
 #endif // DISABLE_GUI
 
 #ifndef DISABLE_WEBUI
     delete m_webui;
 #endif
 
+    delete RSS::AutoDownloader::instance();
+    delete RSS::Session::instance();
+
     ScanFoldersModel::freeInstance();
     BitTorrent::Session::freeInstance();
 #ifndef DISABLE_COUNTRIES_RESOLUTION
     Net::GeoIPManager::freeInstance();
 #endif
     Net::DownloadManager::freeInstance();
+    Net::ProxyConfigurationManager::freeInstance();
     Preferences::freeInstance();
     SettingsStorage::freeInstance();
     delete m_fileLogger;
     Logger::freeInstance();
     IconProvider::freeInstance();
+    Utils::Fs::removeDirRecursive(Utils::Fs::tempPath());
 
 #ifndef DISABLE_GUI
+    if (m_window) {
 #ifdef Q_OS_WIN
-    typedef BOOL (WINAPI *PSHUTDOWNBRDESTROY)(HWND);
-    PSHUTDOWNBRDESTROY shutdownBRDestroy = (PSHUTDOWNBRDESTROY)::GetProcAddress(::GetModuleHandleW(L"User32.dll"), "ShutdownBlockReasonDestroy");
-    // Only available on Vista+
-    if (shutdownBRDestroy)
-        shutdownBRDestroy((HWND)m_window->effectiveWinId());
+        typedef BOOL (WINAPI *PSHUTDOWNBRDESTROY)(HWND);
+        PSHUTDOWNBRDESTROY shutdownBRDestroy = (PSHUTDOWNBRDESTROY)::GetProcAddress(::GetModuleHandleW(L"User32.dll"), "ShutdownBlockReasonDestroy");
+        // Only available on Vista+
+        if (shutdownBRDestroy)
+            shutdownBRDestroy((HWND)m_window->effectiveWinId());
 #endif // Q_OS_WIN
-    delete m_window;
+        delete m_window;
+    }
 #endif // DISABLE_GUI
 
     if (m_shutdownAct != ShutdownDialogAction::Exit) {
@@ -643,3 +728,12 @@ void Application::cleanup()
         Utils::Misc::shutdownComputer(m_shutdownAct);
     }
 }
+
+void Application::validateCommandLineParameters()
+{
+    if (m_commandLineArgs.portableMode && !m_commandLineArgs.profileDir.isEmpty())
+        throw CommandLineParameterError(tr("Portable mode and explicit profile directory options are mutually exclusive"));
+
+    if (m_commandLineArgs.portableMode && m_commandLineArgs.relativeFastresumePaths)
+        Logger::instance()->addMessage(tr("Portable mode implies relative fastresume"), Log::WARNING);
+}
