@@ -41,7 +41,7 @@ InfoHash::InfoHash(const libtorrent::sha1_hash &nativeHash)
     , m_nativeHash(nativeHash)
 {
     char out[(libtorrent::sha1_hash::size * 2) + 1];
-    libtorrent::to_hex((char const*)&m_nativeHash[0], libtorrent::sha1_hash::size, out);
+    libtorrent::to_hex(reinterpret_cast<const char*>(&m_nativeHash[0]), libtorrent::sha1_hash::size, out);
     m_hashString = QString(out);
 }
 
@@ -51,7 +51,7 @@ InfoHash::InfoHash(const QString &hashString)
 {
     QByteArray raw = m_hashString.toLatin1();
     if (raw.size() == 40)
-        m_valid = libtorrent::from_hex(raw.constData(), 40, (char*)&m_nativeHash[0]);
+        m_valid = libtorrent::from_hex(raw.constData(), 40, reinterpret_cast<char*>(&m_nativeHash[0]));
 }
 
 
