@@ -4,7 +4,10 @@
 #include <QObject>
 #include <QTimer>
 
-namespace BitTorrent { class Session; }
+namespace BitTorrent
+{
+    class Session;
+}
 
 class Statistics : QObject
 {
@@ -30,8 +33,8 @@ private:
     // Will overflow at 15.9 EiB
     quint64 m_alltimeUL;
     quint64 m_alltimeDL;
-    qint64 m_sessionUL;
-    qint64 m_sessionDL;
+    quint64 m_sessionUL;
+    quint64 m_sessionDL;
     mutable qint64 m_lastWrite;
     mutable bool m_dirty;
 
