@@ -91,6 +91,7 @@ namespace BitTorrent
         QByteArray metadata() const;
         QStringList filesForPiece(int pieceIndex) const;
         QVector<int> fileIndicesForPiece(int pieceIndex) const;
+        QVector<QByteArray> pieceHashes() const;
 
         using PieceRange = IndexRange<int>;
         // returns pair of the first and the last pieces into which
@@ -100,6 +101,9 @@ namespace BitTorrent
 
         void renameFile(uint index, const QString &newPath);
 
+        bool hasRootFolder() const;
+        void stripRootFolder();
+
         NativePtr nativeInfo() const;
 
     private:
