@@ -29,32 +29,48 @@
 #ifndef HTTP_TYPES_H
 #define HTTP_TYPES_H
 
-#include <QString>
-#include <QMap>
 #include <QHostAddress>
+#include <QString>
 #include <QVector>
 
 #include "base/types.h"
 
 namespace Http
 {
-    const QString HEADER_SET_COOKIE = "Set-Cookie";
-    const QString HEADER_CONTENT_TYPE = "Content-Type";
-    const QString HEADER_CONTENT_ENCODING = "Content-Encoding";
-    const QString HEADER_CONTENT_LENGTH = "Content-Length";
-    const QString HEADER_CACHE_CONTROL = "Cache-Control";
+    const char HEADER_CACHE_CONTROL[] = "cache-control";
+    const char HEADER_CONTENT_ENCODING[] = "content-encoding";
+    const char HEADER_CONTENT_LENGTH[] = "content-length";
+    const char HEADER_CONTENT_SECURITY_POLICY[] = "content-security-policy";
+    const char HEADER_CONTENT_TYPE[] = "content-type";
+    const char HEADER_DATE[] = "date";
+    const char HEADER_HOST[] = "host";
+    const char HEADER_ORIGIN[] = "origin";
+    const char HEADER_REFERER[] = "referer";
+    const char HEADER_SET_COOKIE[] = "set-cookie";
+    const char HEADER_X_CONTENT_TYPE_OPTIONS[] = "x-content-type-options";
+    const char HEADER_X_FORWARDED_HOST[] = "x-forwarded-host";
+    const char HEADER_X_FRAME_OPTIONS[] = "x-frame-options";
+    const char HEADER_X_XSS_PROTECTION[] = "x-xss-protection";
 
-    const QString CONTENT_TYPE_CSS = "text/css; charset=UTF-8";
-    const QString CONTENT_TYPE_GIF = "image/gif";
-    const QString CONTENT_TYPE_HTML = "text/html; charset=UTF-8";
-    const QString CONTENT_TYPE_JS = "application/javascript; charset=UTF-8";
-    const QString CONTENT_TYPE_JSON = "application/json";
-    const QString CONTENT_TYPE_PNG = "image/png";
-    const QString CONTENT_TYPE_TXT = "text/plain; charset=UTF-8";
+    const char CONTENT_TYPE_CSS[] = "text/css; charset=UTF-8";
+    const char CONTENT_TYPE_GIF[] = "image/gif";
+    const char CONTENT_TYPE_HTML[] = "text/html; charset=UTF-8";
+    const char CONTENT_TYPE_JS[] = "application/javascript; charset=UTF-8";
+    const char CONTENT_TYPE_JSON[] = "application/json";
+    const char CONTENT_TYPE_PNG[] = "image/png";
+    const char CONTENT_TYPE_TXT[] = "text/plain; charset=UTF-8";
+    const char CONTENT_TYPE_SVG[] = "image/svg+xml";
+
+    // portability: "\r\n" doesn't guarantee mapping to the correct value
+    const char CRLF[] = {0x0D, 0x0A, '\0'};
 
     struct Environment
     {
+        QHostAddress localAddress;
+        quint16 localPort;
+
         QHostAddress clientAddress;
+        quint16 clientPort;
     };
 
     struct UploadedFile
