@@ -1,6 +1,7 @@
 #include "logger.h"
 
 #include <QDateTime>
+#include "base/utils/string.h"
 
 Logger* Logger::m_instance = 0;
 
@@ -36,7 +37,7 @@ void Logger::addMessage(const QString &message, const Log::MsgType &type)
 {
     QWriteLocker locker(&lock);
 
-    Log::Msg temp = { msgCounter++, QDateTime::currentMSecsSinceEpoch(), type, message };
+    Log::Msg temp = { msgCounter++, QDateTime::currentMSecsSinceEpoch(), type, message.toHtmlEscaped() };
     m_messages.push_back(temp);
 
     if (m_messages.size() >= MAX_LOG_MESSAGES)
@@ -49,7 +50,7 @@ void Logger::addPeer(const QString &ip, bool blocked, const QString &reason)
 {
     QWriteLocker locker(&lock);
 
-    Log::Peer temp = { peerCounter++, QDateTime::currentMSecsSinceEpoch(), ip, blocked, reason };
+    Log::Peer temp = { peerCounter++, QDateTime::currentMSecsSinceEpoch(), ip.toHtmlEscaped(), blocked, reason.toHtmlEscaped() };
     m_peers.push_back(temp);
 
     if (m_peers.size() >= MAX_LOG_MESSAGES)
@@ -89,3 +90,8 @@ QVector<Log::Peer> Logger::getPeers(int lastKnownId) const
 
     return m_peers.mid(size - diff);
 }
+
+void LogMsg(const QString &message, const Log::MsgType &type)
+{
+    Logger::instance()->addMessage(message, type);
+}
