@@ -1,6 +1,6 @@
 /*
- * Bittorrent Client using Qt4 and libtorrent.
- * Copyright (C) 2011  Christophe Dumez
+ * Bittorrent Client using Qt and libtorrent.
+ * Copyright (C) 2011  Christophe Dumez <chris@qbittorrent.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -24,20 +24,16 @@
  * modify file(s), you may extend this exception to your version of the file(s),
  * but you are not obligated to do so. If you do not wish to do so, delete this
  * exception statement from your version.
- *
- * Contact : chris@qbittorrent.org
  */
 
 #include <QDebug>
 #include <QRegExp>
 #include <QStringList>
-#ifdef QBT_USES_QT5
 #include <QUrlQuery>
-#endif
 
 #include "base/logger.h"
-#include "base/net/downloadmanager.h"
 #include "base/net/downloadhandler.h"
+#include "base/net/downloadmanager.h"
 #include "dnsupdater.h"
 
 using namespace Net;
@@ -80,7 +76,7 @@ void DNSUpdater::checkPublicIP()
 
     DownloadHandler *handler = DownloadManager::instance()->downloadUrl(
                 "http://checkip.dyndns.org", false, 0, false,
-                QString("qBittorrent/%1").arg(VERSION));
+                "qBittorrent/" QBT_VERSION_2);
     connect(handler, SIGNAL(downloadFinished(QString, QByteArray)), SLOT(ipRequestFinished(QString, QByteArray)));
     connect(handler, SIGNAL(downloadFailed(QString, QString)), SLOT(ipRequestFailed(QString, QString)));
 
@@ -127,7 +123,7 @@ void DNSUpdater::updateDNSService()
     m_lastIPCheckTime = QDateTime::currentDateTime();
     DownloadHandler *handler = DownloadManager::instance()->downloadUrl(
                 getUpdateUrl(), false, 0, false,
-                QString("qBittorrent/%1").arg(VERSION));
+                "qBittorrent/" QBT_VERSION_2);
     connect(handler, SIGNAL(downloadFinished(QString, QByteArray)), SLOT(ipUpdateFinished(QString, QByteArray)));
     connect(handler, SIGNAL(downloadFailed(QString, QString)), SLOT(ipUpdateFailed(QString, QString)));
 }
@@ -145,7 +141,7 @@ QString DNSUpdater::getUpdateUrl() const
 
     Q_ASSERT(!m_lastIP.isNull());
     // Service specific
-    switch(m_service) {
+    switch (m_service) {
     case DNS::DYNDNS:
         url.setHost("members.dyndns.org");
         break;
@@ -158,15 +154,10 @@ QString DNSUpdater::getUpdateUrl() const
     }
     url.setPath("/nic/update");
 
-#ifndef QBT_USES_QT5
-    url.addQueryItem("hostname", m_domain);
-    url.addQueryItem("myip", m_lastIP.toString());
-#else
     QUrlQuery urlQuery(url);
     urlQuery.addQueryItem("hostname", m_domain);
     urlQuery.addQueryItem("myip", m_lastIP.toString());
     url.setQuery(urlQuery);
-#endif
     Q_ASSERT(url.isValid());
 
     qDebug() << Q_FUNC_INFO << url.toString();
@@ -237,7 +228,6 @@ void DNSUpdater::processIPUpdateReply(const QString &reply)
     if (code == "abuse") {
         logger->addMessage(tr("Dynamic DNS error: Your username was blocked due to abuse."), Log::CRITICAL);
         m_state = FATAL;
-        return;
     }
 }
 
@@ -296,7 +286,7 @@ void DNSUpdater::updateCredentials()
 
 QUrl DNSUpdater::getRegistrationUrl(int service)
 {
-    switch(service) {
+    switch (service) {
     case DNS::DYNDNS:
         return QUrl("https://www.dyndns.com/account/services/hosts/add.html");
     case DNS::NOIP:
