@@ -27,36 +27,37 @@
  * exception statement from your version.
  */
 
-#include <QDebug>
-#include <QFile>
-#include <QDir>
-#include <QHostAddress>
+#include "geoipmanager.h"
+
 #include <QDateTime>
+#include <QDebug>
+#include <QDir>
+#include <QFile>
+#include <QHostAddress>
 
 #include "base/logger.h"
 #include "base/preferences.h"
+#include "base/profile.h"
 #include "base/utils/fs.h"
 #include "base/utils/gzip.h"
-#include "downloadmanager.h"
 #include "downloadhandler.h"
+#include "downloadmanager.h"
 #include "private/geoipdatabase.h"
-#include "geoipmanager.h"
 
 static const char DATABASE_URL[] = "https://geolite.maxmind.com/download/geoip/database/GeoLite2-Country.mmdb.gz";
 static const char GEOIP_FOLDER[] = "GeoIP";
 static const char GEOIP_FILENAME[] = "GeoLite2-Country.mmdb";
-static const int CACHE_SIZE = 1000;
 static const int UPDATE_INTERVAL = 30; // Days between database updates
 
 using namespace Net;
 
 // GeoIPManager
 
-GeoIPManager *GeoIPManager::m_instance = 0;
+GeoIPManager *GeoIPManager::m_instance = nullptr;
 
 GeoIPManager::GeoIPManager()
     : m_enabled(false)
-    , m_geoIPDatabase(0)
+    , m_geoIPDatabase(nullptr)
 {
     configure();
     connect(Preferences::instance(), SIGNAL(changed()), SLOT(configure()));
@@ -78,7 +79,7 @@ void GeoIPManager::freeInstance()
 {
     if (m_instance) {
         delete m_instance;
-        m_instance = 0;
+        m_instance = nullptr;
     }
 }
 
@@ -91,11 +92,11 @@ void GeoIPManager::loadDatabase()
 {
     if (m_geoIPDatabase) {
         delete m_geoIPDatabase;
-        m_geoIPDatabase = 0;
+        m_geoIPDatabase = nullptr;
     }
 
     QString filepath = Utils::Fs::expandPathAbs(
-                QString("%1%2/%3").arg(Utils::Fs::QDesktopServicesDataLocation())
+                QString("%1%2/%3").arg(specialFolderLocation(SpecialFolder::Data))
                 .arg(GEOIP_FOLDER).arg(GEOIP_FILENAME));
 
     QString error;
@@ -407,7 +408,7 @@ void GeoIPManager::configure()
         }
         else if (!m_enabled && m_geoIPDatabase) {
             delete m_geoIPDatabase;
-            m_geoIPDatabase = 0;
+            m_geoIPDatabase = nullptr;
         }
     }
 }
@@ -416,8 +417,10 @@ void GeoIPManager::downloadFinished(const QString &url, QByteArray data)
 {
     Q_UNUSED(url);
 
-    if (!Utils::Gzip::uncompress(data, data)) {
-        Logger::instance()->addMessage(tr("Could not uncompress GeoIP database file."), Log::WARNING);
+    bool ok = false;
+    data = Utils::Gzip::decompress(data, &ok);
+    if (!ok) {
+        Logger::instance()->addMessage(tr("Could not decompress GeoIP database file."), Log::WARNING);
         return;
     }
 
@@ -432,7 +435,7 @@ void GeoIPManager::downloadFinished(const QString &url, QByteArray data)
                                            .arg(m_geoIPDatabase->type()).arg(m_geoIPDatabase->buildEpoch().toString()),
                                            Log::INFO);
             QString targetPath = Utils::Fs::expandPathAbs(
-                        Utils::Fs::QDesktopServicesDataLocation() + GEOIP_FOLDER);
+                        specialFolderLocation(SpecialFolder::Data) + GEOIP_FOLDER);
             if (!QDir(targetPath).exists())
                 QDir().mkpath(targetPath);
             QFile targetFile(QString("%1/%2").arg(targetPath).arg(GEOIP_FILENAME));
