@@ -29,8 +29,8 @@
 #ifndef NET_PORTFORWARDER_H
 #define NET_PORTFORWARDER_H
 
-#include <QObject>
 #include <QHash>
+#include <QObject>
 
 namespace libtorrent
 {
@@ -49,14 +49,14 @@ namespace Net
         static void freeInstance();
         static PortForwarder *instance();
 
-        void addPort(qint16 port);
-        void deletePort(qint16 port);
+        bool isEnabled() const;
+        void setEnabled(bool enabled);
 
-    private slots:
-        void configure();
+        void addPort(quint16 port);
+        void deletePort(quint16 port);
 
     private:
-        explicit PortForwarder(libtorrent::session *const provider, QObject *parent = 0);
+        explicit PortForwarder(libtorrent::session *const provider, QObject *parent = nullptr);
         ~PortForwarder();
 
         void start();
@@ -64,7 +64,7 @@ namespace Net
 
         bool m_active;
         libtorrent::session *m_provider;
-        QHash<qint16, int> m_mappedPorts;
+        QHash<quint16, int> m_mappedPorts;
 
         static PortForwarder *m_instance;
     };
