@@ -33,14 +33,18 @@
 #ifndef PREFERENCES_H
 #define PREFERENCES_H
 
-#include <QTime>
 #include <QDateTime>
+#include <QHostAddress>
 #include <QList>
-#include <QTimer>
-#include <QReadWriteLock>
 #include <QNetworkCookie>
+#include <QReadWriteLock>
+#include <QSize>
+#include <QStringList>
+#include <QTime>
+#include <QTimer>
 #include <QVariant>
 
+#include "base/utils/net.h"
 #include "types.h"
 
 enum scheduler_days
@@ -57,18 +61,6 @@ enum scheduler_days
     SUN
 };
 
-namespace Proxy
-{
-    enum ProxyType
-    {
-        HTTP = 1,
-        SOCKS5 = 2,
-        HTTP_PW = 3,
-        SOCKS5_PW = 4,
-        SOCKS4 = 5
-    };
-}
-
 namespace TrayIcon
 {
     enum Style
@@ -101,8 +93,7 @@ class Preferences: public QObject
     const QVariant value(const QString &key, const QVariant &defaultValue = QVariant()) const;
     void setValue(const QString &key, const QVariant &value);
 
-    static Preferences* m_instance;
-    int m_randomPort;
+    static Preferences *m_instance;
 
 signals:
     void changed();
@@ -110,7 +101,7 @@ signals:
 public:
     static void initInstance();
     static void freeInstance();
-    static Preferences* instance();
+    static Preferences *instance();
 
     // General options
     QString getLocale() const;
@@ -127,16 +118,10 @@ public:
     void setHideZeroValues(bool b);
     int getHideZeroComboValues() const;
     void setHideZeroComboValues(int n);
-    bool useRandomPort() const;
-    void setRandomPort(bool b);
-    bool systrayIntegration() const;
-    void setSystrayIntegration(bool enabled);
+    bool isStatusbarDisplayed() const;
+    void setStatusbarDisplayed(bool displayed);
     bool isToolbarDisplayed() const;
     void setToolbarDisplayed(bool displayed);
-    bool minimizeToTray() const;
-    void setMinimizeToTray(bool b);
-    bool closeToTray() const;
-    void setCloseToTray(bool b);
     bool startMinimized() const;
     void setStartMinimized(bool b);
     bool isSplashScreenDisabled() const;
@@ -149,24 +134,16 @@ public:
 #endif
 
     // Downloads
-    bool useIncompleteFilesExtension() const;
-    void useIncompleteFilesExtension(bool enabled);
     QString lastLocationPath() const;
     void setLastLocationPath(const QString &path);
-    bool preAllocateAllFiles() const;
-    void preAllocateAllFiles(bool enabled);
     QVariantHash getScanDirs() const;
     void setScanDirs(const QVariantHash &dirs);
     QString getScanDirsLastPath() const;
     void setScanDirsLastPath(const QString &path);
-    bool isTorrentExportEnabled() const;
-    QString getTorrentExportDir() const;
-    void setTorrentExportDir(QString path);
-    bool isFinishedTorrentExportEnabled() const;
-    QString getFinishedTorrentExportDir() const;
-    void setFinishedTorrentExportDir(QString path);
     bool isMailNotificationEnabled() const;
     void setMailNotificationEnabled(bool enabled);
+    QString getMailNotificationSender() const;
+    void setMailNotificationSender(const QString &mail);
     QString getMailNotificationEmail() const;
     void setMailNotificationEmail(const QString &mail);
     QString getMailNotificationSMTP() const;
@@ -185,22 +162,6 @@ public:
     void setActionOnDblClOnTorrentFn(int act);
 
     // Connection options
-    int getSessionPort() const;
-    void setSessionPort(int port);
-    bool isUPnPEnabled() const;
-    void setUPnPEnabled(bool enabled);
-    int getGlobalDownloadLimit() const;
-    void setGlobalDownloadLimit(int limit);
-    int getGlobalUploadLimit() const;
-    void setGlobalUploadLimit(int limit);
-    int getAltGlobalDownloadLimit() const;
-    void setAltGlobalDownloadLimit(int limit);
-    int getAltGlobalUploadLimit() const;
-    void setAltGlobalUploadLimit(int limit);
-    bool isAltBandwidthEnabled() const;
-    void setAltBandwidthEnabled(bool enabled);
-    bool isSchedulerEnabled() const;
-    void setSchedulerEnabled(bool enabled);
     QTime getSchedulerStartTime() const;
     void setSchedulerStartTime(const QTime &time);
     QTime getSchedulerEndTime() const;
@@ -208,98 +169,43 @@ public:
     scheduler_days getSchedulerDays() const;
     void setSchedulerDays(scheduler_days days);
 
-    // Proxy options
-    bool isProxyEnabled() const;
-    bool isProxyAuthEnabled() const;
-    void setProxyAuthEnabled(bool enabled);
-    QString getProxyIp() const;
-    void setProxyIp(const QString &ip);
-    unsigned short getProxyPort() const;
-    void setProxyPort(unsigned short port);
-    QString getProxyUsername() const;
-    void setProxyUsername(const QString &username);
-    QString getProxyPassword() const;
-    void setProxyPassword(const QString &password);
-    int getProxyType() const;
-    void setProxyType(int type);
-    bool proxyPeerConnections() const;
-    void setProxyPeerConnections(bool enabled);
-    bool getForceProxy() const;
-    void setForceProxy(bool enabled);
-    void setProxyOnlyForTorrents(bool enabled);
-    bool isProxyOnlyForTorrents() const;
-
-    // Bittorrent options
-    int getMaxConnecs() const;
-    void setMaxConnecs(int val);
-    int getMaxConnecsPerTorrent() const;
-    void setMaxConnecsPerTorrent(int val);
-    int getMaxUploads() const;
-    void setMaxUploads(int val);
-    int getMaxUploadsPerTorrent() const;
-    void setMaxUploadsPerTorrent(int val);
-    bool isuTPEnabled() const;
-    void setuTPEnabled(bool enabled);
-    bool isuTPRateLimited() const;
-    void setuTPRateLimited(bool enabled);
-    bool isDHTEnabled() const;
-    void setDHTEnabled(bool enabled);
-    bool isPeXEnabled() const;
-    void setPeXEnabled(bool enabled);
-    bool isLSDEnabled() const;
-    void setLSDEnabled(bool enabled);
-    int getEncryptionSetting() const;
-    void setEncryptionSetting(int val);
-    bool isAddTrackersEnabled() const;
-    void setAddTrackersEnabled(bool enabled);
-    QString getTrackersList() const;
-    void setTrackersList(const QString &val);
-    qreal getGlobalMaxRatio() const;
-    void setGlobalMaxRatio(qreal ratio);
-
-    // IP Filter
-    bool isFilteringEnabled() const;
-    void setFilteringEnabled(bool enabled);
-    bool isFilteringTrackerEnabled() const;
-    void setFilteringTrackerEnabled(bool enabled);
-    QString getFilter() const;
-    void setFilter(const QString &path);
-    QStringList bannedIPs() const;
-    void banIP(const QString &ip);
-
     // Search
     bool isSearchEnabled() const;
     void setSearchEnabled(bool enabled);
 
-    // Queueing system
-    bool isQueueingSystemEnabled() const;
-    void setQueueingSystemEnabled(bool enabled);
-    int getMaxActiveDownloads() const;
-    void setMaxActiveDownloads(int val);
-    int getMaxActiveUploads() const;
-    void setMaxActiveUploads(int val);
-    int getMaxActiveTorrents() const;
-    void setMaxActiveTorrents(int val);
-    bool ignoreSlowTorrentsForQueueing() const;
-    void setIgnoreSlowTorrentsForQueueing(bool ignore);
+    // HTTP Server
     bool isWebUiEnabled() const;
     void setWebUiEnabled(bool enabled);
-    bool isWebUiLocalAuthEnabled() const;
-    void setWebUiLocalAuthEnabled(bool enabled);
+    QString getServerDomains() const;
+    void setServerDomains(const QString &str);
+    QString getWebUiAddress() const;
+    void setWebUiAddress(const QString &addr);
     quint16 getWebUiPort() const;
     void setWebUiPort(quint16 port);
     bool useUPnPForWebUIPort() const;
     void setUPnPForWebUIPort(bool enabled);
+
+    // Authentication
+    bool isWebUiLocalAuthEnabled() const;
+    void setWebUiLocalAuthEnabled(bool enabled);
+    bool isWebUiAuthSubnetWhitelistEnabled() const;
+    void setWebUiAuthSubnetWhitelistEnabled(bool enabled);
+    QList<Utils::Net::Subnet> getWebUiAuthSubnetWhitelist() const;
+    void setWebUiAuthSubnetWhitelist(const QList<Utils::Net::Subnet> &subnets);
     QString getWebUiUsername() const;
     void setWebUiUsername(const QString &username);
     QString getWebUiPassword() const;
     void setWebUiPassword(const QString &new_password);
+
+    // HTTPS
     bool isWebUiHttpsEnabled() const;
     void setWebUiHttpsEnabled(bool enabled);
     QByteArray getWebUiHttpsCertificate() const;
     void setWebUiHttpsCertificate(const QByteArray &data);
     QByteArray getWebUiHttpsKey() const;
     void setWebUiHttpsKey(const QByteArray &data);
+
+    // Dynamic DNS
     bool isDynDNSEnabled() const;
     void setDynDNSEnabled(bool enabled);
     DNS::Service getDynDNSService() const;
@@ -331,50 +237,12 @@ public:
     void setShutdownqBTWhenDownloadsComplete(bool shutdown);
     bool dontConfirmAutoExit() const;
     void setDontConfirmAutoExit(bool dontConfirmAutoExit);
-    uint diskCacheSize() const;
-    void setDiskCacheSize(uint size);
-    uint diskCacheTTL() const;
-    void setDiskCacheTTL(uint ttl);
-    bool osCache() const;
-    void setOsCache(bool enable);
-    uint saveResumeDataInterval() const;
-    void setSaveResumeDataInterval(uint m);
-    uint outgoingPortsMin() const;
-    void setOutgoingPortsMin(uint val);
-    uint outgoingPortsMax() const;
-    void setOutgoingPortsMax(uint val);
-    bool getIgnoreLimitsOnLAN() const;
-    void setIgnoreLimitsOnLAN(bool ignore);
-    bool includeOverheadInLimits() const;
-    void includeOverheadInLimits(bool include);
-    bool trackerExchangeEnabled() const;
-    void setTrackerExchangeEnabled(bool enable);
     bool recheckTorrentsOnCompletion() const;
     void recheckTorrentsOnCompletion(bool recheck);
-    unsigned int getRefreshInterval() const;
-    void setRefreshInterval(uint interval);
     bool resolvePeerCountries() const;
     void resolvePeerCountries(bool resolve);
     bool resolvePeerHostNames() const;
     void resolvePeerHostNames(bool resolve);
-    int getMaxHalfOpenConnections() const;
-    void setMaxHalfOpenConnections(int value);
-    QString getNetworkInterface() const;
-    void setNetworkInterface(const QString& iface);
-    QString getNetworkInterfaceName() const;
-    void setNetworkInterfaceName(const QString& iface);
-    QString getNetworkInterfaceAddress() const;
-    void setNetworkInterfaceAddress(const QString& addr);
-    bool getListenIPv6() const;
-    void setListenIPv6(bool enable);
-    QString getNetworkAddress() const;
-    void setNetworkAddress(const QString& addr);
-    bool isAnonymousModeEnabled() const;
-    void enableAnonymousMode(bool enabled);
-    bool isSuperSeedingEnabled() const;
-    void enableSuperSeeding(bool enabled);
-    bool announceToAllTrackers() const;
-    void setAnnounceToAllTrackers(bool enabled);
 #if (defined(Q_OS_UNIX) && !defined(Q_OS_MAC))
     bool useSystemIconTheme() const;
     void useSystemIconTheme(bool enabled);
@@ -396,8 +264,6 @@ public:
     static void setTorrentFileAssoc();
     static void setMagnetLinkAssoc();
 #endif
-    bool isTrackerEnabled() const;
-    void setTrackerEnabled(bool enabled);
     int getTrackerPort() const;
     void setTrackerPort(int port);
 #if defined(Q_OS_WIN) || defined(Q_OS_MAC)
@@ -408,8 +274,18 @@ public:
     void setConfirmTorrentDeletion(bool enabled);
     bool confirmTorrentRecheck() const;
     void setConfirmTorrentRecheck(bool enabled);
+    bool confirmRemoveAllTags() const;
+    void setConfirmRemoveAllTags(bool enabled);
+#ifndef Q_OS_MAC
+    bool systrayIntegration() const;
+    void setSystrayIntegration(bool enabled);
+    bool minimizeToTray() const;
+    void setMinimizeToTray(bool b);
+    bool closeToTray() const;
+    void setCloseToTray(bool b);
     TrayIcon::Style trayIconStyle() const;
     void setTrayIconStyle(TrayIcon::Style style);
+#endif
 
     // Stuff that don't appear in the Options GUI but are saved
     // in the same file.
@@ -425,12 +301,8 @@ public:
     void setMainVSplitterState(const QByteArray &state);
     QString getMainLastDir() const;
     void setMainLastDir(const QString &path);
-#ifndef DISABLE_GUI
     QSize getPrefSize(const QSize &defaultSize) const;
     void setPrefSize(const QSize &size);
-#endif
-    QPoint getPrefPos() const;
-    void setPrefPos(const QPoint &pos);
     QStringList getPrefHSplitterSizes() const;
     void setPrefHSplitterSizes(const QStringList &sizes);
     QByteArray getPeerListState() const;
@@ -445,36 +317,27 @@ public:
     void setPropVisible(const bool visible);
     QByteArray getPropTrackerListState() const;
     void setPropTrackerListState(const QByteArray &state);
-    QByteArray getRssGeometry() const;
-    void setRssGeometry(const QByteArray &geometry);
+    QSize getRssGeometrySize(const QSize &defaultSize) const;
+    void setRssGeometrySize(const QSize &geometry);
     QByteArray getRssHSplitterSizes() const;
     void setRssHSplitterSizes(const QByteArray &sizes);
     QStringList getRssOpenFolders() const;
     void setRssOpenFolders(const QStringList &folders);
-    QByteArray getRssHSplitterState() const;
-    void setRssHSplitterState(const QByteArray &state);
-    QByteArray getRssVSplitterState() const;
-    void setRssVSplitterState(const QByteArray &state);
-    QString getSearchColsWidth() const;
-    void setSearchColsWidth(const QString &width);
+    QByteArray getRssSideSplitterState() const;
+    void setRssSideSplitterState(const QByteArray &state);
+    QByteArray getRssMainSplitterState() const;
+    void setRssMainSplitterState(const QByteArray &state);
+    QByteArray getSearchTabHeaderState() const;
+    void setSearchTabHeaderState(const QByteArray &state);
     QStringList getSearchEngDisabled() const;
     void setSearchEngDisabled(const QStringList &engines);
-    QString getCreateTorLastAddPath() const;
-    void setCreateTorLastAddPath(const QString &path);
-    QString getCreateTorLastSavePath() const;
-    void setCreateTorLastSavePath(const QString &path);
-    QString getCreateTorTrackers() const;
-    void setCreateTorTrackers(const QString &path);
-    QByteArray getCreateTorGeometry() const;
-    void setCreateTorGeometry(const QByteArray &geometry);
-    bool getCreateTorIgnoreRatio() const;
-    void setCreateTorIgnoreRatio(const bool ignore);
     QString getTorImportLastContentDir() const;
     void setTorImportLastContentDir(const QString &path);
     QByteArray getTorImportGeometry() const;
     void setTorImportGeometry(const QByteArray &geometry);
     bool getStatusFilterState() const;
     bool getCategoryFilterState() const;
+    bool getTagFilterState() const;
     bool getTrackerFilterState() const;
     int getTransSelFilter() const;
     void setTransSelFilter(const int &index);
@@ -483,19 +346,9 @@ public:
     int getToolbarTextPosition() const;
     void setToolbarTextPosition(const int position);
 
-    //From old RssSettings class
-    bool isRSSEnabled() const;
-    void setRSSEnabled(const bool enabled);
-    uint getRSSRefreshInterval() const;
-    void setRSSRefreshInterval(const uint &interval);
-    int getRSSMaxArticlesPerFeed() const;
-    void setRSSMaxArticlesPerFeed(const int &nb);
-    bool isRssDownloadingEnabled() const;
-    void setRssDownloadingEnabled(const bool b);
-    QStringList getRssFeedsUrls() const;
-    void setRssFeedsUrls(const QStringList &rssFeeds);
-    QStringList getRssFeedsAliases() const;
-    void setRssFeedsAliases(const QStringList &rssAliases);
+    // From old RssSettings class
+    bool isRSSWidgetEnabled() const;
+    void setRSSWidgetVisible(const bool enabled);
 
     // Network
     QList<QNetworkCookie> getNetworkCookies() const;
@@ -512,6 +365,7 @@ public:
 public slots:
     void setStatusFilterState(bool checked);
     void setCategoryFilterState(bool checked);
+    void setTagFilterState(bool checked);
     void setTrackerFilterState(bool checked);
 
     void apply();
