@@ -36,10 +36,7 @@ typedef QSet<QString> QStringSet;
 
 namespace BitTorrent
 {
-
-class TorrentHandle;
-class TorrentState;
-
+    class TorrentHandle;
 }
 
 class TorrentFilter
@@ -58,8 +55,10 @@ public:
         Errored
     };
 
+    // These mean any permutation, including no category / tag.
     static const QString AnyCategory;
     static const QStringSet AnyHash;
+    static const QString AnyTag; 
 
     static const TorrentFilter DownloadingTorrent;
     static const TorrentFilter SeedingTorrent;
@@ -71,14 +70,16 @@ public:
     static const TorrentFilter ErroredTorrent;
 
     TorrentFilter();
-    // category: pass empty string for "no category" or null string (QString()) for "any category"
-    TorrentFilter(Type type, QStringSet hashSet = AnyHash, QString category = AnyCategory);
-    TorrentFilter(QString filter, QStringSet hashSet = AnyHash, QString category = AnyCategory);
+    // category & tags: pass empty string for uncategorized / untagged torrents.
+    // Pass null string (QString()) to disable filtering (i.e. all torrents).
+    TorrentFilter(const Type type, const QStringSet &hashSet = AnyHash, const QString &category = AnyCategory, const QString &tag = AnyTag);
+    TorrentFilter(const QString &filter, const QStringSet &hashSet = AnyHash, const QString &category = AnyCategory, const QString &tags = AnyTag);
 
     bool setType(Type type);
     bool setTypeByName(const QString &filter);
     bool setHashSet(const QStringSet &hashSet);
     bool setCategory(const QString &category);
+    bool setTag(const QString &tag);
 
     bool match(BitTorrent::TorrentHandle *const torrent) const;
 
@@ -86,9 +87,11 @@ private:
     bool matchState(BitTorrent::TorrentHandle *const torrent) const;
     bool matchHash(BitTorrent::TorrentHandle *const torrent) const;
     bool matchCategory(BitTorrent::TorrentHandle *const torrent) const;
+    bool matchTag(BitTorrent::TorrentHandle *const torrent) const;
 
     Type m_type;
     QString m_category;
+    QString m_tag;
     QStringSet m_hashSet;
 };
 
