@@ -35,9 +35,11 @@
 // we put all problematic UTF-8 chars/strings in this file.
 // See issue #3059 for more details (https://github.com/qbittorrent/qBittorrent/issues/3059).
 const char C_INFINITY[] = "∞";
+const char C_NON_BREAKING_SPACE[] = " ";
 const char C_UP[] = "▲";
 const char C_DOWN[] = "▼";
 const char C_COPYRIGHT[] = "©";
+const char C_THIN_SPACE[] = " ";
 const char C_UTP[] = "μTP";
 const char C_LOCALE_ENGLISH[] = "English";
 const char C_LOCALE_ENGLISH_AUSTRALIA[] = "English(Australia)";
@@ -46,16 +48,20 @@ const char C_LOCALE_ESPERANTO[] = "Esperanto";
 const char C_LOCALE_FRENCH[] = "Français";
 const char C_LOCALE_GERMAN[] = "Deutsch";
 const char C_LOCALE_HUNGARIAN[] = "Magyar";
+const char C_LOCALE_ICELANDIC[] = "Íslenska";
 const char C_LOCALE_INDONESIAN[] = "Bahasa Indonesia";
 const char C_LOCALE_ITALIAN[] = "Italiano";
 const char C_LOCALE_DUTCH[] = "Nederlands";
 const char C_LOCALE_SPANISH[] = "Español";
 const char C_LOCALE_CATALAN[] = "Català";
 const char C_LOCALE_GALICIAN[] = "Galego";
+const char C_LOCALE_OCCITAN[] = "lenga d'òc";
 const char C_LOCALE_PORTUGUESE[] = "Português";
 const char C_LOCALE_PORTUGUESE_BRAZIL[] = "Português brasileiro";
 const char C_LOCALE_POLISH[] = "Polski";
+const char C_LOCALE_LATVIAN[] = "latviešu valoda";
 const char C_LOCALE_LITHUANIAN[] = "Lietuvių";
+const char C_LOCALE_MALAY[] = "بهاس ملايو";
 const char C_LOCALE_CZECH[] = "Čeština";
 const char C_LOCALE_SLOVAK[] = "Slovenčina";
 const char C_LOCALE_SLOVENIAN[] = "Slovenščina";
@@ -71,6 +77,7 @@ const char C_LOCALE_NORWEGIAN[] = "Norsk";
 const char C_LOCALE_DANISH[] = "Dansk";
 const char C_LOCALE_BULGARIAN[] = "Български";
 const char C_LOCALE_UKRAINIAN[] = "Українська";
+const char C_LOCALE_UZBEK[] = "أۇزبېك‎";
 const char C_LOCALE_RUSSIAN[] = "Русский";
 const char C_LOCALE_JAPANESE[] = "日本語";
 const char C_LOCALE_HEBREW[] = "עברית";
