@@ -41,32 +41,27 @@ namespace Utils
 {
     namespace Fs
     {
-        QString toNativePath(const QString& path);
-        QString fromNativePath(const QString& path);
-        QString fileExtension(const QString& filename);
-        QString fileName(const QString& file_path);
-        QString folderName(const QString& file_path);
-        qint64 computePathSize(const QString& path);
-        bool sameFiles(const QString& path1, const QString& path2);
-        QString toValidFileSystemName(const QString &name, bool allowSeparators = false);
-        bool isValidFileSystemName(const QString& name, bool allowSeparators = false);
-        qlonglong freeDiskSpaceOnPath(QString path);
-        QString branchPath(const QString& file_path, QString* removed = 0);
-        bool sameFileNames(const QString& first, const QString& second);
-        QString expandPath(const QString& path);
-        QString expandPathAbs(const QString& path);
+        QString toNativePath(const QString &path);
+        QString fromNativePath(const QString &path);
+        QString fileExtension(const QString &filename);
+        QString fileName(const QString &filePath);
+        QString folderName(const QString &filePath);
+        qint64 computePathSize(const QString &path);
+        bool sameFiles(const QString &path1, const QString &path2);
+        QString toValidFileSystemName(const QString &name, bool allowSeparators = false
+                , const QString &pad = QLatin1String(" "));
+        bool isValidFileSystemName(const QString &name, bool allowSeparators = false);
+        qint64 freeDiskSpaceOnPath(const QString &path);
+        QString branchPath(const QString &filePath, QString *removed = 0);
+        bool sameFileNames(const QString &first, const QString &second);
+        QString expandPath(const QString &path);
+        QString expandPathAbs(const QString &path);
 
-        bool smartRemoveEmptyFolderTree(const QString& path);
-        bool forceRemove(const QString& file_path);
-        void removeDirRecursive(const QString& dirName);
+        bool smartRemoveEmptyFolderTree(const QString &path);
+        bool forceRemove(const QString &filePath);
+        void removeDirRecursive(const QString &path);
 
-        /* Ported from Qt4 to drop dependency on QtGui */
-        QString QDesktopServicesDataLocation();
-        QString QDesktopServicesCacheLocation();
-        QString QDesktopServicesDownloadLocation();
-        /* End of Qt4 code */
-
-        QString cacheLocation();
+        QString tempPath();
     }
 }
 
