@@ -32,11 +32,10 @@
 #include <cmath>
 
 #include <QByteArray>
-#include <QtGlobal>
-#include <QLocale>
-#ifdef QBT_USES_QT5
 #include <QCollator>
-#endif
+#include <QLocale>
+#include <QRegExp>
+#include <QtGlobal>
 #ifdef Q_OS_MAC
 #include <QThreadStorage>
 #endif
@@ -46,141 +45,107 @@ namespace
     class NaturalCompare
     {
     public:
-        explicit NaturalCompare(const bool caseSensitive = true)
-            : m_caseSensitive(caseSensitive)
+        explicit NaturalCompare(const Qt::CaseSensitivity caseSensitivity = Qt::CaseSensitive)
+            : m_caseSensitivity(caseSensitivity)
         {
-#ifdef QBT_USES_QT5
-#if defined(Q_OS_WIN)
+#ifdef Q_OS_WIN
             // Without ICU library, QCollator uses the native API on Windows 7+. But that API
             // sorts older versions of μTorrent differently than the newer ones because the
             // 'μ' character is encoded differently and the native API can't cope with that.
             // So default to using our custom natural sorting algorithm instead.
             // See #5238 and #5240
-            // Without ICU library, QCollator doesn't support `setNumericMode(true)` on OS older than Win7
-            // if (QSysInfo::windowsVersion() < QSysInfo::WV_WINDOWS7)
-                return;
-#endif
+            // Without ICU library, QCollator doesn't support `setNumericMode(true)` on an OS older than Win7
+#else
             m_collator.setNumericMode(true);
-            m_collator.setCaseSensitivity(caseSensitive ? Qt::CaseSensitive : Qt::CaseInsensitive);
+            m_collator.setCaseSensitivity(caseSensitivity);
 #endif
         }
 
-        bool operator()(const QString &left, const QString &right) const
+        int operator()(const QString &left, const QString &right) const
         {
-#ifdef QBT_USES_QT5
-#if defined(Q_OS_WIN)
-            // Without ICU library, QCollator uses the native API on Windows 7+. But that API
-            // sorts older versions of μTorrent differently than the newer ones because the
-            // 'μ' character is encoded differently and the native API can't cope with that.
-            // So default to using our custom natural sorting algorithm instead.
-            // See #5238 and #5240
-            // Without ICU library, QCollator doesn't support `setNumericMode(true)` on OS older than Win7
-            // if (QSysInfo::windowsVersion() < QSysInfo::WV_WINDOWS7)
-                return lessThan(left, right);
-#endif
-            return (m_collator.compare(left, right) < 0);
+#ifdef Q_OS_WIN
+            return compare(left, right);
 #else
-            return lessThan(left, right);
+            return m_collator.compare(left, right);
 #endif
         }
 
-        bool lessThan(const QString &left, const QString &right) const
-        {
-            // Return value `false` indicates `right` should go before `left`, otherwise, after
-            int posL = 0;
-            int posR = 0;
-            while (true) {
-                while (true) {
-                    if ((posL == left.size()) || (posR == right.size()))
-                        return (left.size() < right.size());  // when a shorter string is another string's prefix, shorter string place before longer string
-
-                    QChar leftChar = m_caseSensitive ? left[posL] : left[posL].toLower();
-                    QChar rightChar = m_caseSensitive ? right[posR] : right[posR].toLower();
-                    if (leftChar == rightChar)
-                        ;  // compare next character
-                    else if (leftChar.isDigit() && rightChar.isDigit())
-                        break; // Both are digits, break this loop and compare numbers
-                    else
-                        return leftChar < rightChar;
-
-                    ++posL;
-                    ++posR;
-                }
-
-                int startL = posL;
-                while ((posL < left.size()) && left[posL].isDigit())
-                    ++posL;
-#ifdef QBT_USES_QT5
-                int numL = left.midRef(startL, posL - startL).toInt();
-#else
-                int numL = left.mid(startL, posL - startL).toInt();
-#endif
-
-                int startR = posR;
-                while ((posR < right.size()) && right[posR].isDigit())
-                    ++posR;
-#ifdef QBT_USES_QT5
-                int numR = right.midRef(startR, posR - startR).toInt();
-#else
-                int numR = right.mid(startR, posR - startR).toInt();
-#endif
-
-                if (numL != numR)
-                    return (numL < numR);
-
-                // Strings + digits do match and we haven't hit string end
-                // Do another round
-            }
-            return false;
-        }
-
     private:
-#ifdef QBT_USES_QT5
+        int compare(const QString &left, const QString &right) const
+        {
+            // Return value <0: `left` is smaller than `right`
+            // Return value >0: `left` is greater than `right`
+            // Return value =0: both strings are equal
+
+            int posL = 0;
+            int posR = 0;
+            while (true) {
+                if ((posL == left.size()) || (posR == right.size()))
+                    return (left.size() - right.size());  // when a shorter string is another string's prefix, shorter string place before longer string
+
+                const QChar leftChar = (m_caseSensitivity == Qt::CaseSensitive) ? left[posL] : left[posL].toLower();
+                const QChar rightChar = (m_caseSensitivity == Qt::CaseSensitive) ? right[posR] : right[posR].toLower();
+                if (leftChar == rightChar) {
+                    // compare next character
+                    ++posL;
+                    ++posR;
+                }
+                else if (leftChar.isDigit() && rightChar.isDigit()) {
+                    // Both are digits, compare the numbers
+                    const auto consumeNumber = [](const QString &str, int &pos) -> int
+                    {
+                        const int start = pos;
+                        while ((pos < str.size()) && str[pos].isDigit())
+                            ++pos;
+                        return str.midRef(start, (pos - start)).toInt();
+                    };
+
+                    const int numL = consumeNumber(left, posL);
+                    const int numR = consumeNumber(right, posR);
+                    if (numL != numR)
+                        return (numL - numR);
+
+                    // String + digits do match and we haven't hit the end of both strings
+                    // then continue to consume the remainings
+                }
+                else {
+                    return (leftChar.unicode() - rightChar.unicode());
+                }
+            }
+        }
+
         QCollator m_collator;
-#endif
-        const bool m_caseSensitive;
+        const Qt::CaseSensitivity m_caseSensitivity;
     };
 }
 
-bool Utils::String::naturalCompareCaseSensitive(const QString &left, const QString &right)
+int Utils::String::naturalCompare(const QString &left, const QString &right, const Qt::CaseSensitivity caseSensitivity)
 {
     // provide a single `NaturalCompare` instance for easy use
     // https://doc.qt.io/qt-5/threads-reentrancy.html
+    if (caseSensitivity == Qt::CaseSensitive) {
 #ifdef Q_OS_MAC  // workaround for Apple xcode: https://stackoverflow.com/a/29929949
+        static QThreadStorage<NaturalCompare> nCmp;
+        if (!nCmp.hasLocalData())
+            nCmp.setLocalData(NaturalCompare(Qt::CaseSensitive));
+        return (nCmp.localData())(left, right);
+#else
+        thread_local NaturalCompare nCmp(Qt::CaseSensitive);
+        return nCmp(left, right);
+#endif
+    }
+
+#ifdef Q_OS_MAC
     static QThreadStorage<NaturalCompare> nCmp;
-    if (!nCmp.hasLocalData()) nCmp.setLocalData(NaturalCompare(true));
+    if (!nCmp.hasLocalData())
+        nCmp.setLocalData(NaturalCompare(Qt::CaseInsensitive));
     return (nCmp.localData())(left, right);
 #else
-    thread_local NaturalCompare nCmp(true);
+    thread_local NaturalCompare nCmp(Qt::CaseInsensitive);
     return nCmp(left, right);
 #endif
 }
 
-bool Utils::String::naturalCompareCaseInsensitive(const QString &left, const QString &right)
-{
-    // provide a single `NaturalCompare` instance for easy use
-    // https://doc.qt.io/qt-5/threads-reentrancy.html
-#ifdef Q_OS_MAC  // workaround for Apple xcode: https://stackoverflow.com/a/29929949
-    static QThreadStorage<NaturalCompare> nCmp;
-    if (!nCmp.hasLocalData()) nCmp.setLocalData(NaturalCompare(false));
-    return (nCmp.localData())(left, right);
-#else
-    thread_local NaturalCompare nCmp(false);
-    return nCmp(left, right);
-#endif
-}
-
-QString Utils::String::fromStdString(const std::string &str)
-{
-    return QString::fromUtf8(str.c_str());
-}
-
-std::string Utils::String::toStdString(const QString &str)
-{
-    QByteArray utf8 = str.toUtf8();
-    return std::string(utf8.constData(), utf8.length());
-}
-
 // to send numbers instead of strings with suffixes
 QString Utils::String::fromDouble(double n, int precision)
 {
@@ -207,3 +172,12 @@ bool Utils::String::slowEquals(const QByteArray &a, const QByteArray &b)
 
     return (diff == 0);
 }
+
+// This is marked as internal in QRegExp.cpp, but is exported. The alternative would be to
+// copy the code from QRegExp::wc2rx().
+QString qt_regexp_toCanonical(const QString &pattern, QRegExp::PatternSyntax patternSyntax);
+
+QString Utils::String::wildcardToRegex(const QString &pattern)
+{
+    return qt_regexp_toCanonical(pattern, QRegExp::Wildcard);
+}
