@@ -30,25 +30,42 @@
 #ifndef UTILS_STRING_H
 #define UTILS_STRING_H
 
-#include <string>
+#include <QString>
 
 class QByteArray;
-class QString;
+class QLatin1String;
 
 namespace Utils
 {
     namespace String
     {
-        QString fromStdString(const std::string &str);
-        std::string toStdString(const QString &str);
         QString fromDouble(double n, int precision);
 
         // Implements constant-time comparison to protect against timing attacks
         // Taken from https://crackstation.net/hashing-security.htm
         bool slowEquals(const QByteArray &a, const QByteArray &b);
 
-        bool naturalCompareCaseSensitive(const QString &left, const QString &right);
-        bool naturalCompareCaseInsensitive(const QString &left, const QString &right);
+        int naturalCompare(const QString &left, const QString &right, const Qt::CaseSensitivity caseSensitivity);
+        template <Qt::CaseSensitivity caseSensitivity>
+        bool naturalLessThan(const QString &left, const QString &right)
+        {
+            return (naturalCompare(left, right, caseSensitivity) < 0);
+        }
+
+        QString wildcardToRegex(const QString &pattern);
+
+        template <typename T>
+        T unquote(const T &str, const QString &quotes = QLatin1String("\""))
+        {
+            if (str.length() < 2) return str;
+
+            for (auto const quote : quotes) {
+                if (str.startsWith(quote) && str.endsWith(quote))
+                    return str.mid(1, str.length() - 2);
+            }
+
+            return str;
+        }
     }
 }
 
