@@ -38,6 +38,7 @@
 
 #include "base/bittorrent/infohash.h"
 #include "base/bittorrent/torrentinfo.h"
+#include "base/bittorrent/addtorrentparams.h"
 
 namespace BitTorrent
 {
@@ -52,30 +53,35 @@ namespace Ui
 class TorrentContentFilterModel;
 class TorrentFileGuard;
 class PropListDelegate;
+template <typename T> class CachedSettingValue;
 
 class AddNewTorrentDialog: public QDialog
 {
     Q_OBJECT
 
 public:
+    static constexpr int minPathHistoryLength = 0;
+    static constexpr int maxPathHistoryLength = 99;
+
     ~AddNewTorrentDialog();
 
     static bool isEnabled();
     static void setEnabled(bool value);
     static bool isTopLevel();
     static void setTopLevel(bool value);
+    static int savePathHistoryLength();
+    static void setSavePathHistoryLength(int value);
 
-    static void show(QString source, QWidget *parent = 0);
+    static void show(QString source, const BitTorrent::AddTorrentParams &inParams, QWidget *parent);
+    static void show(QString source, QWidget *parent);
 
 private slots:
     void showAdvancedSettings(bool show);
-    void displayContentTreeMenu(const QPoint&);
+    void displayContentTreeMenu(const QPoint &);
     void updateDiskSpaceLabel();
-    void onSavePathChanged(int);
+    void onSavePathChanged(const QString &newPath);
     void renameSelectedFile();
-    void setdialogPosition();
     void updateMetadata(const BitTorrent::TorrentInfo &info);
-    void browseButton_clicked();
     void handleDownloadFailed(const QString &url, const QString &reason);
     void handleRedirectedToMagnet(const QString &url, const QString &magnetUri);
     void handleDownloadFinished(const QString &url, const QString &filePath);
@@ -87,18 +93,19 @@ private slots:
     void reject() override;
 
 private:
-    explicit AddNewTorrentDialog(QWidget *parent = 0);
+    explicit AddNewTorrentDialog(const BitTorrent::AddTorrentParams &inParams, QWidget *parent);
     bool loadTorrent(const QString &torrentPath);
     bool loadMagnet(const BitTorrent::MagnetUri &magnetUri);
     void populateSavePathComboBox();
     void saveSavePathHistory() const;
-    int indexOfSavePath(const QString& save_path);
+    int indexOfSavePath(const QString &save_path);
     void loadState();
     void saveState();
     void setMetadataProgressIndicator(bool visibleIndicator, const QString &labelText = QString());
     void setupTreeview();
-    QString defaultSavePath() const;
     void setCommentText(const QString &str) const;
+    void setSavePath(const QString &newPath);
+    static CachedSettingValue<int> &savePathHistoryLengthSetting();
 
     void showEvent(QShowEvent *event) override;
 
@@ -109,10 +116,10 @@ private:
     QString m_filePath;
     BitTorrent::InfoHash m_hash;
     BitTorrent::TorrentInfo m_torrentInfo;
-    QShortcut *editHotkey;
     QByteArray m_headerState;
     int m_oldIndex;
     QScopedPointer<TorrentFileGuard> m_torrentGuard;
+    BitTorrent::AddTorrentParams m_torrentParams;
 };
 
 #endif // ADDNEWTORRENTDIALOG_H
