@@ -45,6 +45,8 @@ namespace
 
 #ifdef Q_OS_MAC
     const QString OS_TYPE("Mac OS X");
+#elif defined(Q_OS_WIN) && (defined(__x86_64__) || defined(_M_X64))
+    const QString OS_TYPE("Windows x64");
 #else
     const QString OS_TYPE("Windows");
 #endif
@@ -64,7 +66,7 @@ void ProgramUpdater::checkForUpdates()
                 RSS_URL, false, 0, false,
                 // Don't change this User-Agent. In case our updater goes haywire,
                 // the filehost can identify it and contact us.
-                QString("qBittorrent/%1 ProgramUpdater (www.qbittorrent.org)").arg(VERSION));
+                "qBittorrent/" QBT_VERSION_2 " ProgramUpdater (www.qbittorrent.org)");
     connect(handler, SIGNAL(downloadFinished(QString,QByteArray)), SLOT(rssDownloadFinished(QString,QByteArray)));
     connect(handler, SIGNAL(downloadFailed(QString,QString)), SLOT(rssDownloadFailed(QString,QString)));
 }
@@ -97,9 +99,9 @@ void ProgramUpdater::rssDownloadFinished(const QString &url, const QByteArray &d
         else if (xml.isEndElement()) {
             if (inItem && xml.name() == "item") {
                 if (type.compare(OS_TYPE, Qt::CaseInsensitive) == 0) {
-                    qDebug("The last update available is %s", qPrintable(version));
+                    qDebug("The last update available is %s", qUtf8Printable(version));
                     if (!version.isEmpty()) {
-                        qDebug("Detected version is %s", qPrintable(version));
+                        qDebug("Detected version is %s", qUtf8Printable(version));
                         if (isVersionMoreRecent(version))
                             m_updateUrl = updateLink;
                     }
@@ -135,9 +137,9 @@ void ProgramUpdater::updateProgram()
 bool ProgramUpdater::isVersionMoreRecent(const QString &remoteVersion) const
 {
     QRegExp regVer("([0-9.]+)");
-    if (regVer.indexIn(QString(VERSION)) >= 0) {
+    if (regVer.indexIn(QBT_VERSION) >= 0) {
         QString localVersion = regVer.cap(1);
-        qDebug() << Q_FUNC_INFO << "local version:" << localVersion << "/" << VERSION;
+        qDebug() << Q_FUNC_INFO << "local version:" << localVersion << "/" << QBT_VERSION;
         QStringList remoteParts = remoteVersion.split('.');
         QStringList localParts = localVersion.split('.');
         for (int i = 0; i<qMin(remoteParts.size(), localParts.size()); ++i) {
@@ -151,7 +153,7 @@ bool ProgramUpdater::isVersionMoreRecent(const QString &remoteVersion) const
             return true;
         // versions are equal, check if the local version is a development release, in which case it is older (2.9.2beta < 2.9.2)
         QRegExp regDevel("(alpha|beta|rc)");
-        if (regDevel.indexIn(VERSION) >= 0)
+        if (regDevel.indexIn(QBT_VERSION) >= 0)
             return true;
     }
     return false;
