@@ -1,6 +1,6 @@
 /*
- * Bittorrent Client using Qt4 and libtorrent.
- * Copyright (C) 2006  Christophe Dumez
+ * Bittorrent Client using Qt and libtorrent.
+ * Copyright (C) 2006  Christophe Dumez <chris@qbittorrent.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -24,35 +24,34 @@
  * modify file(s), you may extend this exception to your version of the file(s),
  * but you are not obligated to do so. If you do not wish to do so, delete this
  * exception statement from your version.
- *
- * Contact : chris@qbittorrent.org
  */
 
-#include <QStandardItemModel>
-#include <QSortFilterProxyModel>
-#include <QSet>
+#include "peerlistwidget.h"
+
+#include <QApplication>
+#include <QClipboard>
 #include <QHeaderView>
 #include <QMenu>
-#include <QClipboard>
 #include <QMessageBox>
-#ifdef QBT_USES_QT5
+#include <QSet>
+#include <QSortFilterProxyModel>
+#include <QStandardItemModel>
 #include <QTableView>
-#endif
+#include <QWheelEvent>
 
-#include "base/net/reverseresolution.h"
-#include "base/bittorrent/torrenthandle.h"
 #include "base/bittorrent/peerinfo.h"
-#include "base/preferences.h"
+#include "base/bittorrent/torrenthandle.h"
 #include "base/logger.h"
-#include "base/unicodestrings.h"
-#include "propertieswidget.h"
 #include "base/net/geoipmanager.h"
-#include "peersadditiondlg.h"
-#include "speedlimitdlg.h"
+#include "base/net/reverseresolution.h"
+#include "base/preferences.h"
+#include "base/unicodestrings.h"
 #include "guiiconprovider.h"
 #include "peerlistdelegate.h"
 #include "peerlistsortmodel.h"
-#include "peerlistwidget.h"
+#include "peersadditiondlg.h"
+#include "propertieswidget.h"
+#include "speedlimitdlg.h"
 
 PeerListWidget::PeerListWidget(PropertiesWidget *parent)
     : QTreeView(parent)
@@ -68,7 +67,7 @@ PeerListWidget::PeerListWidget(PropertiesWidget *parent)
     setSelectionMode(QAbstractItemView::ExtendedSelection);
     header()->setStretchLastSection(false);
     // List Model
-    m_listModel = new QStandardItemModel(0, PeerListDelegate::COL_COUNT);
+    m_listModel = new QStandardItemModel(0, PeerListDelegate::COL_COUNT, this);
     m_listModel->setHeaderData(PeerListDelegate::COUNTRY, Qt::Horizontal, tr("Country")); // Country flag column
     m_listModel->setHeaderData(PeerListDelegate::IP, Qt::Horizontal, tr("IP"));
     m_listModel->setHeaderData(PeerListDelegate::PORT, Qt::Horizontal, tr("Port"));
@@ -82,8 +81,16 @@ PeerListWidget::PeerListWidget(PropertiesWidget *parent)
     m_listModel->setHeaderData(PeerListDelegate::TOT_UP, Qt::Horizontal, tr("Uploaded", "i.e: total data uploaded"));
     m_listModel->setHeaderData(PeerListDelegate::RELEVANCE, Qt::Horizontal, tr("Relevance", "i.e: How relevant this peer is to us. How many pieces it has that we don't."));
     m_listModel->setHeaderData(PeerListDelegate::DOWNLOADING_PIECE, Qt::Horizontal, tr("Files", "i.e. files that are being downloaded right now"));
+    // Set header text alignment
+    m_listModel->setHeaderData(PeerListDelegate::PORT, Qt::Horizontal, QVariant(Qt::AlignRight | Qt::AlignVCenter), Qt::TextAlignmentRole);
+    m_listModel->setHeaderData(PeerListDelegate::PROGRESS, Qt::Horizontal, QVariant(Qt::AlignRight | Qt::AlignVCenter), Qt::TextAlignmentRole);
+    m_listModel->setHeaderData(PeerListDelegate::DOWN_SPEED, Qt::Horizontal, QVariant(Qt::AlignRight | Qt::AlignVCenter), Qt::TextAlignmentRole);
+    m_listModel->setHeaderData(PeerListDelegate::UP_SPEED, Qt::Horizontal, QVariant(Qt::AlignRight | Qt::AlignVCenter), Qt::TextAlignmentRole);
+    m_listModel->setHeaderData(PeerListDelegate::TOT_DOWN, Qt::Horizontal, QVariant(Qt::AlignRight | Qt::AlignVCenter), Qt::TextAlignmentRole);
+    m_listModel->setHeaderData(PeerListDelegate::TOT_UP, Qt::Horizontal, QVariant(Qt::AlignRight | Qt::AlignVCenter), Qt::TextAlignmentRole);
+    m_listModel->setHeaderData(PeerListDelegate::RELEVANCE, Qt::Horizontal, QVariant(Qt::AlignRight | Qt::AlignVCenter), Qt::TextAlignmentRole);
     // Proxy model to support sorting without actually altering the underlying model
-    m_proxyModel = new PeerListSortModel();
+    m_proxyModel = new PeerListSortModel(this);
     m_proxyModel->setDynamicSortFilter(true);
     m_proxyModel->setSourceModel(m_listModel);
     m_proxyModel->setSortCaseSensitivity(Qt::CaseInsensitive);
@@ -93,7 +100,7 @@ PeerListWidget::PeerListWidget(PropertiesWidget *parent)
     m_resolveCountries = Preferences::instance()->resolvePeerCountries();
     if (!m_resolveCountries)
         hideColumn(PeerListDelegate::COUNTRY);
-    //Ensure that at least one column is visible at all times
+    // Ensure that at least one column is visible at all times
     bool atLeastOne = false;
     for (unsigned int i = 0; i < PeerListDelegate::IP_HIDDEN; i++) {
         if (!isColumnHidden(i)) {
@@ -103,9 +110,9 @@ PeerListWidget::PeerListWidget(PropertiesWidget *parent)
     }
     if (!atLeastOne)
         setColumnHidden(PeerListDelegate::IP, false);
-    //To also mitigate the above issue, we have to resize each column when
-    //its size is 0, because explicitly 'showing' the column isn't enough
-    //in the above scenario.
+    // To also mitigate the above issue, we have to resize each column when
+    // its size is 0, because explicitly 'showing' the column isn't enough
+    // in the above scenario.
     for (unsigned int i = 0; i < PeerListDelegate::IP_HIDDEN; i++)
         if ((columnWidth(i) <= 0) && !isColumnHidden(i))
             resizeColumnToContents(i);
@@ -121,37 +128,31 @@ PeerListWidget::PeerListWidget(PropertiesWidget *parent)
     updatePeerHostNameResolutionState();
     // SIGNAL/SLOT
     header()->setContextMenuPolicy(Qt::CustomContextMenu);
-    connect(header(), SIGNAL(customContextMenuRequested(const QPoint &)), this, SLOT(displayToggleColumnsMenu(const QPoint &)));
+    connect(header(), SIGNAL(customContextMenuRequested(const QPoint&)), this, SLOT(displayToggleColumnsMenu(const QPoint&)));
     connect(header(), SIGNAL(sectionClicked(int)), SLOT(handleSortColumnChanged(int)));
     handleSortColumnChanged(header()->sortIndicatorSection());
     m_copyHotkey = new QShortcut(QKeySequence::Copy, this, SLOT(copySelectedPeers()), 0, Qt::WidgetShortcut);
 
-#ifdef QBT_USES_QT5
     // This hack fixes reordering of first column with Qt5.
     // https://github.com/qtproject/qtbase/commit/e0fc088c0c8bc61dbcaf5928b24986cd61a22777
     QTableView unused;
     unused.setVerticalHeader(this->header());
     this->header()->setParent(this);
     unused.setVerticalHeader(new QHeaderView(Qt::Horizontal));
-#endif
 }
 
 PeerListWidget::~PeerListWidget()
 {
     saveSettings();
-    delete m_proxyModel;
-    delete m_listModel;
-    delete m_listDelegate;
     if (m_resolver)
         delete m_resolver;
-    delete m_copyHotkey;
 }
 
-void PeerListWidget::displayToggleColumnsMenu(const QPoint&)
+void PeerListWidget::displayToggleColumnsMenu(const QPoint &)
 {
     QMenu hideshowColumn(this);
     hideshowColumn.setTitle(tr("Column visibility"));
-    QList<QAction*> actions;
+    QList<QAction *> actions;
     for (int i = 0; i < PeerListDelegate::IP_HIDDEN; ++i) {
         if ((i == PeerListDelegate::COUNTRY) && !Preferences::instance()->resolvePeerCountries()) {
             actions.append(nullptr); // keep the index in sync
@@ -192,7 +193,7 @@ void PeerListWidget::updatePeerHostNameResolutionState()
     if (Preferences::instance()->resolvePeerHostNames()) {
         if (!m_resolver) {
             m_resolver = new Net::ReverseResolution(this);
-            connect(m_resolver, SIGNAL(ipResolved(QString, QString)), SLOT(handleResolved(QString, QString)));
+            connect(m_resolver, SIGNAL(ipResolved(QString,QString)), SLOT(handleResolved(QString,QString)));
             loadPeers(m_properties->getCurrentTorrent(), true);
         }
     }
@@ -217,7 +218,7 @@ void PeerListWidget::updatePeerCountryResolutionState()
     }
 }
 
-void PeerListWidget::showPeerListMenu(const QPoint&)
+void PeerListWidget::showPeerListMenu(const QPoint &)
 {
     QMenu menu;
     bool emptyMenu = true;
@@ -230,8 +231,8 @@ void PeerListWidget::showPeerListMenu(const QPoint&)
         addPeerAct = menu.addAction(GuiIconProvider::instance()->getIcon("user-group-new"), tr("Add a new peer..."));
         emptyMenu = false;
     }
-    QAction *banAct = 0;
-    QAction *copyPeerAct = 0;
+    QAction *banAct = nullptr;
+    QAction *copyPeerAct = nullptr;
     if (!selectionModel()->selectedRows().isEmpty()) {
         copyPeerAct = menu.addAction(GuiIconProvider::instance()->getIcon("edit-copy"), tr("Copy IP:port"));
         menu.addSeparator();
@@ -240,24 +241,25 @@ void PeerListWidget::showPeerListMenu(const QPoint&)
     }
     if (emptyMenu) return;
     QAction *act = menu.exec(QCursor::pos());
-    if (act == 0) return;
+    if (!act) return;
+
     if (act == addPeerAct) {
-        QList<BitTorrent::PeerAddress> peersList = PeersAdditionDlg::askForPeers();
+        QList<BitTorrent::PeerAddress> peersList = PeersAdditionDlg::askForPeers(this);
         int peerCount = 0;
         foreach (const BitTorrent::PeerAddress &addr, peersList) {
             if (torrent->connectPeer(addr)) {
-                qDebug("Adding peer %s...", qPrintable(addr.ip.toString()));
+                qDebug("Adding peer %s...", qUtf8Printable(addr.ip.toString()));
                 Logger::instance()->addMessage(tr("Manually adding peer '%1'...").arg(addr.ip.toString()));
-                peerCount++;
+                ++peerCount;
             }
             else {
                 Logger::instance()->addMessage(tr("The peer '%1' could not be added to this torrent.").arg(addr.ip.toString()), Log::WARNING);
             }
         }
         if (peerCount < peersList.length())
-            QMessageBox::information(0, tr("Peer addition"), tr("Some peers could not be added. Check the Log for details."));
+            QMessageBox::information(this, tr("Peer addition"), tr("Some peers could not be added. Check the Log for details."));
         else if (peerCount > 0)
-            QMessageBox::information(0, tr("Peer addition"), tr("The peers were added to this torrent."));
+            QMessageBox::information(this, tr("Peer addition"), tr("The peers were added to this torrent."));
         return;
     }
     if (act == banAct) {
@@ -276,8 +278,7 @@ void PeerListWidget::banSelectedPeers()
     int ret = QMessageBox::question(this, tr("Ban peer permanently"), tr("Are you sure you want to ban permanently the selected peers?"),
                                     tr("&Yes"), tr("&No"),
                                     QString(), 0, 1);
-    if (ret)
-        return;
+    if (ret) return;
 
     QModelIndexList selectedIndexes = selectionModel()->selectedRows();
     foreach (const QModelIndex &index, selectedIndexes) {
@@ -361,7 +362,7 @@ void PeerListWidget::loadPeers(BitTorrent::TorrentHandle *const torrent, bool fo
     // Delete peers that are gone
     QSetIterator<QString> it(oldeersSet);
     while (it.hasNext()) {
-        const QString& ip = it.next();
+        const QString &ip = it.next();
         m_missingFlags.remove(ip);
         m_peerAddresses.remove(ip);
         QStandardItem *item = m_peerItems.take(ip);
@@ -369,7 +370,7 @@ void PeerListWidget::loadPeers(BitTorrent::TorrentHandle *const torrent, bool fo
     }
 }
 
-QStandardItem* PeerListWidget::addPeer(const QString& ip, BitTorrent::TorrentHandle *const torrent, const BitTorrent::PeerInfo &peer)
+QStandardItem *PeerListWidget::addPeer(const QString &ip, BitTorrent::TorrentHandle *const torrent, const BitTorrent::PeerInfo &peer)
 {
     int row = m_listModel->rowCount();
     // Adding Peer to peer list
@@ -392,7 +393,7 @@ QStandardItem* PeerListWidget::addPeer(const QString& ip, BitTorrent::TorrentHan
     m_listModel->setData(m_listModel->index(row, PeerListDelegate::CONNECTION), peer.connectionType());
     m_listModel->setData(m_listModel->index(row, PeerListDelegate::FLAGS), peer.flags());
     m_listModel->setData(m_listModel->index(row, PeerListDelegate::FLAGS), peer.flagsDescription(), Qt::ToolTipRole);
-    m_listModel->setData(m_listModel->index(row, PeerListDelegate::CLIENT), peer.client());
+    m_listModel->setData(m_listModel->index(row, PeerListDelegate::CLIENT), peer.client().toHtmlEscaped());
     m_listModel->setData(m_listModel->index(row, PeerListDelegate::PROGRESS), peer.progress());
     m_listModel->setData(m_listModel->index(row, PeerListDelegate::DOWN_SPEED), peer.payloadDownSpeed());
     m_listModel->setData(m_listModel->index(row, PeerListDelegate::UP_SPEED), peer.payloadUpSpeed());
@@ -423,7 +424,7 @@ void PeerListWidget::updatePeer(const QString &ip, BitTorrent::TorrentHandle *co
     m_listModel->setData(m_listModel->index(row, PeerListDelegate::PORT), peer.address().port);
     m_listModel->setData(m_listModel->index(row, PeerListDelegate::FLAGS), peer.flags());
     m_listModel->setData(m_listModel->index(row, PeerListDelegate::FLAGS), peer.flagsDescription(), Qt::ToolTipRole);
-    m_listModel->setData(m_listModel->index(row, PeerListDelegate::CLIENT), peer.client());
+    m_listModel->setData(m_listModel->index(row, PeerListDelegate::CLIENT), peer.client().toHtmlEscaped());
     m_listModel->setData(m_listModel->index(row, PeerListDelegate::PROGRESS), peer.progress());
     m_listModel->setData(m_listModel->index(row, PeerListDelegate::DOWN_SPEED), peer.payloadDownSpeed());
     m_listModel->setData(m_listModel->index(row, PeerListDelegate::UP_SPEED), peer.payloadUpSpeed());
@@ -439,7 +440,7 @@ void PeerListWidget::handleResolved(const QString &ip, const QString &hostname)
 {
     QStandardItem *item = m_peerItems.value(ip, 0);
     if (item) {
-        qDebug("Resolved %s -> %s", qPrintable(ip), qPrintable(hostname));
+        qDebug("Resolved %s -> %s", qUtf8Printable(ip), qUtf8Printable(hostname));
         item->setData(hostname, Qt::DisplayRole);
     }
 }
@@ -455,3 +456,16 @@ void PeerListWidget::handleSortColumnChanged(int col)
     }
 }
 
+void PeerListWidget::wheelEvent(QWheelEvent *event)
+{
+    event->accept();
+
+    if (event->modifiers() & Qt::ShiftModifier) {
+        // Shift + scroll = horizontal scroll
+        QWheelEvent scrollHEvent(event->pos(), event->globalPos(), event->delta(), event->buttons(), event->modifiers(), Qt::Horizontal);
+        QTreeView::wheelEvent(&scrollHEvent);
+        return;
+    }
+
+    QTreeView::wheelEvent(event);  // event delegated to base class
+}
