@@ -1,6 +1,6 @@
 /*
  * Bittorrent Client using Qt and libtorrent.
- * Copyright (C) 2006  Christophe Dumez
+ * Copyright (C) 2006  Christophe Dumez <chris@qbittorrent.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -24,35 +24,47 @@
  * modify file(s), you may extend this exception to your version of the file(s),
  * but you are not obligated to do so. If you do not wish to do so, delete this
  * exception statement from your version.
- *
- * Contact : chris@qbittorrent.org
  */
 
-#include <QStyleOptionProgressBarV2>
-#include <QStyleOptionViewItemV2>
-#include <QStyleOptionComboBox>
+#include "proplistdelegate.h"
+
+#include <QApplication>
 #include <QComboBox>
 #include <QModelIndex>
 #include <QPainter>
+#include <QPalette>
 #include <QProgressBar>
-#include <QApplication>
+#include <QStyleOptionProgressBar>
 
 #ifdef Q_OS_WIN
-#ifndef QBT_USES_QT5
-#include <QPlastiqueStyle>
-#else
 #include <QProxyStyle>
 #endif
-#endif
 
+#include "base/unicodestrings.h"
 #include "base/utils/misc.h"
 #include "base/utils/string.h"
 #include "propertieswidget.h"
-#include "proplistdelegate.h"
 #include "torrentcontentmodelitem.h"
 
-PropListDelegate::PropListDelegate(PropertiesWidget *properties, QObject *parent)
-    : QItemDelegate(parent)
+namespace
+{
+
+    QPalette progressBarDisabledPalette()
+    {
+        auto getPalette = []() {
+                              QProgressBar bar;
+                              bar.setEnabled(false);
+                              QStyleOptionProgressBar opt;
+                              opt.initFrom(&bar);
+                              return opt.palette;
+                          };
+        static QPalette palette = getPalette();
+        return palette;
+    }
+}
+
+PropListDelegate::PropListDelegate(PropertiesWidget *properties)
+    : QItemDelegate(properties)
     , m_properties(properties)
 {
 }
@@ -60,52 +72,43 @@ PropListDelegate::PropListDelegate(PropertiesWidget *properties, QObject *parent
 void PropListDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const
 {
     painter->save();
-    QStyleOptionViewItemV2 opt = QItemDelegate::setOptions(index, option);
+    QStyleOptionViewItem opt = QItemDelegate::setOptions(index, option);
+    QItemDelegate::drawBackground(painter, opt, index);
 
-    switch(index.column()) {
+    switch (index.column()) {
     case PCSIZE:
-        QItemDelegate::drawBackground(painter, opt, index);
+    case REMAINING:
         QItemDelegate::drawDisplay(painter, opt, option.rect, Utils::Misc::friendlyUnit(index.data().toLongLong()));
         break;
-    case REMAINING:
-        QItemDelegate::drawBackground(painter, opt, index);
-        if (index.sibling(index.row(), PRIORITY).data().toInt() == prio::IGNORED) {
-            QItemDelegate::drawDisplay(painter, opt, option.rect, tr("N/A"));
-        }
-        else {
-            QItemDelegate::drawDisplay(painter, opt, option.rect, Utils::Misc::friendlyUnit(index.data().toLongLong()));
-        }
-        break;
-    case PROGRESS:
-        if (index.data().toDouble() >= 0) {
-            QStyleOptionProgressBarV2 newopt;
+    case PROGRESS: {
+            if (index.data().toDouble() < 0)
+                break;
+
+            QStyleOptionProgressBar newopt;
             qreal progress = index.data().toDouble() * 100.;
             newopt.rect = opt.rect;
             newopt.text = ((progress == 100.0) ? QString("100%") : Utils::String::fromDouble(progress, 1) + "%");
-            newopt.progress = (int)progress;
+            newopt.progress = int(progress);
             newopt.maximum = 100;
             newopt.minimum = 0;
-            newopt.state |= QStyle::State_Enabled;
             newopt.textVisible = true;
+            if (index.sibling(index.row(), PRIORITY).data().toInt() == prio::IGNORED) {
+                newopt.state &= ~QStyle::State_Enabled;
+                newopt.palette = progressBarDisabledPalette();
+            }
+            else {
+                newopt.state |= QStyle::State_Enabled;
+            }
+
 #ifndef Q_OS_WIN
             QApplication::style()->drawControl(QStyle::CE_ProgressBar, &newopt, painter);
 #else
             // XXX: To avoid having the progress text on the right of the bar
-#ifndef QBT_USES_QT5
-            QPlastiqueStyle st;
-#else
-            QProxyStyle st("fusion");
+            QProxyStyle("fusion").drawControl(QStyle::CE_ProgressBar, &newopt, painter, 0);
 #endif
-            st.drawControl(QStyle::CE_ProgressBar, &newopt, painter, 0);
-#endif
-        }
-        else {
-            // Do not display anything if the file is disabled (progress  == 0)
-            QItemDelegate::drawBackground(painter, opt, index);
         }
         break;
     case PRIORITY: {
-            QItemDelegate::drawBackground(painter, opt, index);
             QString text = "";
             switch (index.data().toInt()) {
             case prio::MIXED:
@@ -127,6 +130,19 @@ void PropListDelegate::paint(QPainter *painter, const QStyleOptionViewItem &opti
             QItemDelegate::drawDisplay(painter, opt, option.rect, text);
         }
         break;
+    case AVAILABILITY: {
+            const qreal availability = index.data().toDouble();
+            if (availability < 0) {
+                QItemDelegate::drawDisplay(painter, opt, option.rect, tr("N/A"));
+            }
+            else {
+                const QString value = (availability >= 1.0)
+                                        ? QLatin1String("100")
+                                        : Utils::String::fromDouble(availability * 100., 1);
+                QItemDelegate::drawDisplay(painter, opt, option.rect, value + C_THIN_SPACE + QLatin1Char('%'));
+            }
+        }
+        break;
     default:
         QItemDelegate::paint(painter, option, index);
         break;
@@ -136,38 +152,40 @@ void PropListDelegate::paint(QPainter *painter, const QStyleOptionViewItem &opti
 
 void PropListDelegate::setEditorData(QWidget *editor, const QModelIndex &index) const
 {
-    QComboBox *combobox = static_cast<QComboBox*>(editor);
+    QComboBox *combobox = static_cast<QComboBox *>(editor);
     // Set combobox index
-    switch(index.data().toInt()) {
-    case prio::HIGH:
-        combobox->setCurrentIndex(1);
+    switch (index.data().toInt()) {
+    case prio::IGNORED:
+        combobox->setCurrentIndex(0);
         break;
-    case prio::MAXIMUM:
+    case prio::HIGH:
         combobox->setCurrentIndex(2);
         break;
+    case prio::MAXIMUM:
+        combobox->setCurrentIndex(3);
+        break;
     default:
-        combobox->setCurrentIndex(0);
+        combobox->setCurrentIndex(1);
         break;
     }
 }
 
 QWidget *PropListDelegate::createEditor(QWidget *parent, const QStyleOptionViewItem &, const QModelIndex &index) const
 {
-    if (index.column() != PRIORITY) return 0;
+    if (index.column() != PRIORITY) return nullptr;
 
     if (m_properties) {
         BitTorrent::TorrentHandle *const torrent = m_properties->getCurrentTorrent();
         if (!torrent || !torrent->hasMetadata() || torrent->isSeed())
-            return 0;
+            return nullptr;
     }
 
-    if (index.data().toInt() <= 0) {
-        // IGNORED or MIXED
-        return 0;
-    }
+    if (index.data().toInt() == prio::MIXED)
+        return nullptr;
 
-    QComboBox* editor = new QComboBox(parent);
+    QComboBox *editor = new QComboBox(parent);
     editor->setFocusPolicy(Qt::StrongFocus);
+    editor->addItem(tr("Do not download", "Do not download (priority)"));
     editor->addItem(tr("Normal", "Normal (priority)"));
     editor->addItem(tr("High", "High (priority)"));
     editor->addItem(tr("Maximum", "Maximum (priority)"));
@@ -176,15 +194,18 @@ QWidget *PropListDelegate::createEditor(QWidget *parent, const QStyleOptionViewI
 
 void PropListDelegate::setModelData(QWidget *editor, QAbstractItemModel *model, const QModelIndex &index) const
 {
-    QComboBox *combobox = static_cast<QComboBox*>(editor);
+    QComboBox *combobox = static_cast<QComboBox *>(editor);
     int value = combobox->currentIndex();
     qDebug("PropListDelegate: setModelData(%d)", value);
 
-    switch(value)  {
-    case 1:
-        model->setData(index, prio::HIGH); // HIGH
+    switch (value) {
+    case 0:
+        model->setData(index, prio::IGNORED); // IGNORED
         break;
     case 2:
+        model->setData(index, prio::HIGH); // HIGH
+        break;
+    case 3:
         model->setData(index, prio::MAXIMUM); // MAX
         break;
     default:
