@@ -1,6 +1,6 @@
 /*
  * Bittorrent Client using Qt and libtorrent.
- * Copyright (C) 2006  Christophe Dumez
+ * Copyright (C) 2006  Christophe Dumez <chris@qbittorrent.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -24,37 +24,42 @@
  * modify file(s), you may extend this exception to your version of the file(s),
  * but you are not obligated to do so. If you do not wish to do so, delete this
  * exception statement from your version.
- *
- * Contact : chris@qbittorrent.org
  */
-
-#include <QStringList>
-#include <QMessageBox>
-#include <QFile>
-#include <QUrl>
-
-#include "base/utils/misc.h"
-#include "base/utils/fs.h"
-#include "base/net/downloadmanager.h"
-#include "base/net/downloadhandler.h"
-#include "base/bittorrent/trackerentry.h"
-#include "base/bittorrent/torrenthandle.h"
-#include "guiiconprovider.h"
 #include "trackersadditiondlg.h"
 
-TrackersAdditionDlg::TrackersAdditionDlg(BitTorrent::TorrentHandle *const torrent, QWidget *parent)
+#include <QFile>
+#include <QMessageBox>
+#include <QStringList>
+#include <QUrl>
+
+#include "base/bittorrent/torrenthandle.h"
+#include "base/bittorrent/trackerentry.h"
+#include "base/net/downloadhandler.h"
+#include "base/net/downloadmanager.h"
+#include "base/utils/fs.h"
+#include "base/utils/misc.h"
+#include "guiiconprovider.h"
+#include "ui_trackersadditiondlg.h"
+
+TrackersAdditionDlg::TrackersAdditionDlg(QWidget *parent, BitTorrent::TorrentHandle *const torrent)
     : QDialog(parent)
+    , m_ui(new Ui::TrackersAdditionDlg())
     , m_torrent(torrent)
 {
-    setupUi(this);
+    m_ui->setupUi(this);
     // Icons
-    uTorrentListButton->setIcon(GuiIconProvider::instance()->getIcon("download"));
+    m_ui->uTorrentListButton->setIcon(GuiIconProvider::instance()->getIcon("download"));
+}
+
+TrackersAdditionDlg::~TrackersAdditionDlg()
+{
+    delete m_ui;
 }
 
 QStringList TrackersAdditionDlg::newTrackers() const
 {
     QStringList cleanTrackers;
-    foreach (QString url, trackers_list->toPlainText().split("\n")) {
+    foreach (QString url, m_ui->trackers_list->toPlainText().split("\n")) {
         url = url.trimmed();
         if (!url.isEmpty())
             cleanTrackers << url;
@@ -64,21 +69,21 @@ QStringList TrackersAdditionDlg::newTrackers() const
 
 void TrackersAdditionDlg::on_uTorrentListButton_clicked()
 {
-    uTorrentListButton->setEnabled(false);
-    Net::DownloadHandler *handler = Net::DownloadManager::instance()->downloadUrl(list_url->text(), true);
+    m_ui->uTorrentListButton->setEnabled(false);
+    Net::DownloadHandler *handler = Net::DownloadManager::instance()->downloadUrl(m_ui->list_url->text(), true);
     connect(handler, SIGNAL(downloadFinished(QString, QString)), this, SLOT(parseUTorrentList(QString, QString)));
     connect(handler, SIGNAL(downloadFailed(QString, QString)), this, SLOT(getTrackerError(QString, QString)));
-    //Just to show that it takes times
+    // Just to show that it takes times
     setCursor(Qt::WaitCursor);
 }
 
 void TrackersAdditionDlg::parseUTorrentList(const QString &, const QString &path)
 {
-    QFile list_file(path);
-    if (!list_file.open(QFile::ReadOnly)) {
+    QFile listFile(path);
+    if (!listFile.open(QFile::ReadOnly)) {
         QMessageBox::warning(this, tr("I/O Error"), tr("Error while trying to open the downloaded file."), QMessageBox::Ok);
         setCursor(Qt::ArrowCursor);
-        uTorrentListButton->setEnabled(true);
+        m_ui->uTorrentListButton->setEnabled(true);
         Utils::Fs::forceRemove(path);
         return;
     }
@@ -86,32 +91,32 @@ void TrackersAdditionDlg::parseUTorrentList(const QString &, const QString &path
     // Load from torrent handle
     QList<BitTorrent::TrackerEntry> existingTrackers = m_torrent->trackers();
     // Load from current user list
-    QStringList tmp = trackers_list->toPlainText().split("\n");
-    foreach (const QString &user_url, tmp) {
-        BitTorrent::TrackerEntry userTracker(user_url);
+    QStringList tmp = m_ui->trackers_list->toPlainText().split("\n");
+    foreach (const QString &userURL, tmp) {
+        BitTorrent::TrackerEntry userTracker(userURL);
         if (!existingTrackers.contains(userTracker))
             existingTrackers << userTracker;
     }
 
     // Add new trackers to the list
-    if (!trackers_list->toPlainText().isEmpty() && !trackers_list->toPlainText().endsWith("\n"))
-        trackers_list->insertPlainText("\n");
+    if (!m_ui->trackers_list->toPlainText().isEmpty() && !m_ui->trackers_list->toPlainText().endsWith("\n"))
+        m_ui->trackers_list->insertPlainText("\n");
     int nb = 0;
-    while (!list_file.atEnd()) {
-        const QByteArray line = list_file.readLine().trimmed();
+    while (!listFile.atEnd()) {
+        const QString line = listFile.readLine().trimmed();
         if (line.isEmpty()) continue;
         BitTorrent::TrackerEntry newTracker(line);
         if (!existingTrackers.contains(newTracker)) {
-            trackers_list->insertPlainText(line + "\n");
+            m_ui->trackers_list->insertPlainText(line + "\n");
             ++nb;
         }
     }
     // Clean up
-    list_file.close();
+    listFile.close();
     Utils::Fs::forceRemove(path);
     //To restore the cursor ...
     setCursor(Qt::ArrowCursor);
-    uTorrentListButton->setEnabled(true);
+    m_ui->uTorrentListButton->setEnabled(true);
     // Display information message if necessary
     if (nb == 0)
         QMessageBox::information(this, tr("No change"), tr("No additional trackers were found."), QMessageBox::Ok);
@@ -121,14 +126,14 @@ void TrackersAdditionDlg::getTrackerError(const QString &, const QString &error)
 {
     //To restore the cursor ...
     setCursor(Qt::ArrowCursor);
-    uTorrentListButton->setEnabled(true);
+    m_ui->uTorrentListButton->setEnabled(true);
     QMessageBox::warning(this, tr("Download error"), tr("The trackers list could not be downloaded, reason: %1").arg(error), QMessageBox::Ok);
 }
 
-QStringList TrackersAdditionDlg::askForTrackers(BitTorrent::TorrentHandle *const torrent)
+QStringList TrackersAdditionDlg::askForTrackers(QWidget *parent, BitTorrent::TorrentHandle *const torrent)
 {
     QStringList trackers;
-    TrackersAdditionDlg dlg(torrent);
+    TrackersAdditionDlg dlg(parent, torrent);
     if (dlg.exec() == QDialog::Accepted)
         return dlg.newTrackers();
 
