@@ -1,6 +1,6 @@
 /*
- * Bittorrent Client using Qt4 and libtorrent.
- * Copyright (C) 2006  Christophe Dumez
+ * Bittorrent Client using Qt and libtorrent.
+ * Copyright (C) 2006  Christophe Dumez <chris@qbittorrent.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -24,64 +24,53 @@
  * modify file(s), you may extend this exception to your version of the file(s),
  * but you are not obligated to do so. If you do not wish to do so, delete this
  * exception statement from your version.
- *
- * Contact : chris@qbittorrent.org
  */
 
 #ifndef STATUSBAR_H
 #define STATUSBAR_H
 
-#include <QObject>
+#include <QStatusBar>
 
-class QStatusBar;
-class QFrame;
 class QLabel;
-class QTimer;
 class QPushButton;
-class QHBoxLayout;
 
 namespace BitTorrent
 {
-    class SessionStatus;
+    struct SessionStatus;
 }
 
-class StatusBar: public QObject
+class StatusBar: public QStatusBar
 {
     Q_OBJECT
+    Q_DISABLE_COPY(StatusBar)
 
 public:
-    StatusBar(QStatusBar *bar);
-    ~StatusBar();
+    StatusBar(QWidget *parent = nullptr);
+    ~StatusBar() override;
 
-    QPushButton* connectionStatusButton() const;
+signals:
+    void alternativeSpeedsButtonClicked();
+    void connectionButtonClicked();
 
 public slots:
     void showRestartRequired();
-    void stopTimer();
-    void refreshStatusBar();
+
+private slots:
+    void refresh();
     void updateAltSpeedsBtn(bool alternative);
-    void toggleAlternativeSpeeds();
     void capDownloadSpeed();
     void capUploadSpeed();
 
 private:
-    QStatusBar *m_bar;
+    void updateConnectionStatus();
+    void updateDHTNodesNumber();
+    void updateSpeedLabels();
+
     QPushButton *m_dlSpeedLbl;
     QPushButton *m_upSpeedLbl;
     QLabel *m_DHTLbl;
-    QFrame *m_statusSep1;
-    QFrame *m_statusSep2;
-    QFrame *m_statusSep3;
-    QFrame *m_statusSep4;
     QPushButton *m_connecStatusLblIcon;
     QPushButton *m_altSpeedsBtn;
-    QTimer *m_refreshTimer;
-    QWidget *m_container;
-    QHBoxLayout *m_layout;
-
-    void updateConnectionStatus(const BitTorrent::SessionStatus &sessionStatus);
-    void updateDHTNodesNumber(const BitTorrent::SessionStatus &sessionStatus);
-    void updateSpeedLabels(const BitTorrent::SessionStatus &sessionStatus);
 };
 
 #endif // STATUSBAR_H
