@@ -29,7 +29,24 @@
  */
 
 #include <QDir>
+#include <QFileIconProvider>
+#include <QFileInfo>
 #include <QIcon>
+#include <QMap>
+
+#if defined(Q_OS_WIN)
+#include <Windows.h>
+#include <Shellapi.h>
+#include <QtWin>
+#else
+#include <QMimeDatabase>
+#include <QMimeType>
+#endif
+
+#if defined Q_OS_WIN || defined Q_OS_MAC
+#define QBT_PIXMAP_CACHE_FOR_FILE_ICONS
+#include <QPixmapCache>
+#endif
 
 #include "guiiconprovider.h"
 #include "base/utils/misc.h"
@@ -38,6 +55,9 @@
 #include "torrentcontentmodelitem.h"
 #include "torrentcontentmodelfolder.h"
 #include "torrentcontentmodelfile.h"
+#ifdef Q_OS_MAC
+#include "macutilities.h"
+#endif
 
 namespace
 {
@@ -47,21 +67,156 @@ namespace
         return cached;
     }
 
-    QIcon getFileIcon()
+    class UnifiedFileIconProvider: public QFileIconProvider
     {
-        static QIcon cached = GuiIconProvider::instance()->getIcon("text-plain");
-        return cached;
+    public:
+        using QFileIconProvider::icon;
+
+        QIcon icon(const QFileInfo &info) const override
+        {
+            Q_UNUSED(info);
+            static QIcon cached = GuiIconProvider::instance()->getIcon("text-plain");
+            return cached;
+        }
+    };
+
+#ifdef QBT_PIXMAP_CACHE_FOR_FILE_ICONS
+    struct Q_DECL_UNUSED PixmapCacheSetup
+    {
+        static const int PixmapCacheForIconsSize = 2 * 1024 * 1024; // 2 MiB for file icons
+
+        PixmapCacheSetup()
+        {
+            QPixmapCache::setCacheLimit(QPixmapCache::cacheLimit() + PixmapCacheForIconsSize);
+        }
+
+        ~PixmapCacheSetup()
+        {
+            Q_ASSERT(QPixmapCache::cacheLimit() > PixmapCacheForIconsSize);
+            QPixmapCache::setCacheLimit(QPixmapCache::cacheLimit() - PixmapCacheForIconsSize);
+        }
+    };
+
+    PixmapCacheSetup pixmapCacheSetup;
+
+    class CachingFileIconProvider: public UnifiedFileIconProvider
+    {
+    public:
+        using QFileIconProvider::icon;
+
+        QIcon icon(const QFileInfo &info) const final override
+        {
+            const QString ext = info.suffix();
+            if (!ext.isEmpty()) {
+                QPixmap cached;
+                if (QPixmapCache::find(ext, &cached)) return QIcon(cached);
+
+                const QPixmap pixmap = pixmapForExtension(ext);
+                if (!pixmap.isNull()) {
+                    QPixmapCache::insert(ext, pixmap);
+                    return QIcon(pixmap);
+                }
+            }
+            return UnifiedFileIconProvider::icon(info);
+        }
+
+    protected:
+        virtual QPixmap pixmapForExtension(const QString &ext) const = 0;
+    };
+#endif
+
+#if defined(Q_OS_WIN)
+    // See QTBUG-25319 for explanation why this is required
+    class WinShellFileIconProvider final: public CachingFileIconProvider
+    {
+        QPixmap pixmapForExtension(const QString &ext) const override
+        {
+            const QString extWithDot = QLatin1Char('.') + ext;
+            SHFILEINFO sfi = { 0 };
+            HRESULT hr = ::SHGetFileInfoW(extWithDot.toStdWString().c_str(),
+                FILE_ATTRIBUTE_NORMAL, &sfi, sizeof(sfi), SHGFI_ICON | SHGFI_USEFILEATTRIBUTES);
+            if (FAILED(hr))
+                return QPixmap();
+
+            QPixmap iconPixmap = QtWin::fromHICON(sfi.hIcon);
+            ::DestroyIcon(sfi.hIcon);
+            return iconPixmap;
+        }
+    };
+#elif defined(Q_OS_MAC)
+    // There is a similar bug on macOS, to be reported to Qt
+    // https://github.com/qbittorrent/qBittorrent/pull/6156#issuecomment-316302615
+    class MacFileIconProvider final: public CachingFileIconProvider
+    {
+        QPixmap pixmapForExtension(const QString &ext) const override
+        {
+            return ::pixmapForExtension(ext, QSize(32, 32));
+        }
+    };
+#else
+    /**
+     * @brief Tests whether QFileIconProvider actually works
+     *
+     * Some QPA plugins do not implement QPlatformTheme::fileIcon(), and
+     * QFileIconProvider::icon() returns empty icons as the result. Here we ask it for
+     * two icons for probably absent files and when both icons are null, we assume that
+     * the current QPA plugin does not implement QPlatformTheme::fileIcon().
+     */
+    bool doesQFileIconProviderWork()
+    {
+        QFileIconProvider provider;
+        const char PSEUDO_UNIQUE_FILE_NAME[] = "/tmp/qBittorrent-test-QFileIconProvider-845eb448-7ad5-4cdb-b764-b3f322a266a9";
+        QIcon testIcon1 = provider.icon(QFileInfo(
+            QLatin1String(PSEUDO_UNIQUE_FILE_NAME) + QLatin1String(".pdf")));
+        QIcon testIcon2 = provider.icon(QFileInfo(
+            QLatin1String(PSEUDO_UNIQUE_FILE_NAME) + QLatin1String(".png")));
+
+        return (!testIcon1.isNull() || !testIcon2.isNull());
     }
+
+    class MimeFileIconProvider: public UnifiedFileIconProvider
+    {
+        using QFileIconProvider::icon;
+
+        QIcon icon(const QFileInfo &info) const override
+        {
+            const QMimeType mimeType = m_db.mimeTypeForFile(info, QMimeDatabase::MatchExtension);
+            QIcon res = QIcon::fromTheme(mimeType.iconName());
+            if (!res.isNull()) {
+                return res;
+            }
+
+            res = QIcon::fromTheme(mimeType.genericIconName());
+            if (!res.isNull()) {
+                return res;
+            }
+
+            return UnifiedFileIconProvider::icon(info);
+        }
+
+    private:
+        QMimeDatabase m_db;
+    };
+#endif
 }
 
 TorrentContentModel::TorrentContentModel(QObject *parent)
     : QAbstractItemModel(parent)
-    , m_rootItem(new TorrentContentModelFolder(QList<QVariant>({ tr("Name"), tr("Size"), tr("Progress"), tr("Download Priority"), tr("Remaining") })))
+    , m_rootItem(new TorrentContentModelFolder(QList<QVariant>({ tr("Name"), tr("Size"), tr("Progress"), tr("Download Priority"), tr("Remaining"), tr("Availability") })))
 {
+#if defined(Q_OS_WIN)
+    m_fileIconProvider = new WinShellFileIconProvider();
+#elif defined(Q_OS_MAC)
+    m_fileIconProvider = new MacFileIconProvider();
+#else
+    static bool doesBuiltInProviderWork = doesQFileIconProviderWork();
+    m_fileIconProvider = doesBuiltInProviderWork ? new QFileIconProvider() : new MimeFileIconProvider();
+#endif
 }
 
 TorrentContentModel::~TorrentContentModel()
 {
+    delete m_fileIconProvider;
     delete m_rootItem;
 }
 
@@ -76,14 +231,15 @@ void TorrentContentModel::updateFilesProgress(const QVector<qreal> &fp)
         m_filesIndex[i]->setProgress(fp[i]);
     // Update folders progress in the tree
     m_rootItem->recalculateProgress();
+    m_rootItem->recalculateAvailability();
     emit dataChanged(index(0, 0), index(rowCount(), columnCount()));
 }
 
 void TorrentContentModel::updateFilesPriorities(const QVector<int> &fprio)
 {
-    Q_ASSERT(m_filesIndex.size() == (int)fprio.size());
+    Q_ASSERT(m_filesIndex.size() == fprio.size());
     // XXX: Why is this necessary?
-    if (m_filesIndex.size() != (int)fprio.size())
+    if (m_filesIndex.size() != fprio.size())
         return;
 
     emit layoutAboutToBeChanged();
@@ -92,6 +248,20 @@ void TorrentContentModel::updateFilesPriorities(const QVector<int> &fprio)
     emit dataChanged(index(0, 0), index(rowCount(), columnCount()));
 }
 
+void TorrentContentModel::updateFilesAvailability(const QVector<qreal> &fa)
+{
+    Q_ASSERT(m_filesIndex.size() == fa.size());
+    // XXX: Why is this necessary?
+    if (m_filesIndex.size() != fa.size()) return;
+
+    emit layoutAboutToBeChanged();
+    for (int i = 0; i < fa.size(); ++i)
+        m_filesIndex[i]->setAvailability(fa[i]);
+    // Update folders progress in the tree
+    m_rootItem->recalculateProgress();
+    emit dataChanged(index(0, 0), index(rowCount(), columnCount()));
+}
+
 QVector<int> TorrentContentModel::getFilePriorities() const
 {
     QVector<int> prio;
@@ -122,9 +292,9 @@ bool TorrentContentModel::setData(const QModelIndex& index, const QVariant& valu
     if (!index.isValid())
         return false;
 
-    if ((index.column() == 0) && (role == Qt::CheckStateRole)) {
+    if ((index.column() == TorrentContentModelItem::COL_NAME) && (role == Qt::CheckStateRole)) {
         TorrentContentModelItem *item = static_cast<TorrentContentModelItem*>(index.internalPointer());
-        qDebug("setData(%s, %d", qPrintable(item->name()), value.toInt());
+        qDebug("setData(%s, %d", qUtf8Printable(item->name()), value.toInt());
         if (item->priority() != value.toInt()) {
             if (value.toInt() == Qt::PartiallyChecked)
                 item->setPriority(prio::MIXED);
@@ -134,6 +304,7 @@ bool TorrentContentModel::setData(const QModelIndex& index, const QVariant& valu
                 item->setPriority(prio::NORMAL);
             // Update folders progress in the tree
             m_rootItem->recalculateProgress();
+            m_rootItem->recalculateAvailability();
             emit dataChanged(this->index(0, 0), this->index(rowCount() - 1, columnCount() - 1));
             emit filteredFilesChanged();
         }
@@ -181,23 +352,26 @@ QVariant TorrentContentModel::data(const QModelIndex& index, int role) const
         return QVariant();
 
     TorrentContentModelItem* item = static_cast<TorrentContentModelItem*>(index.internalPointer());
-    if ((index.column() == 0) && (role == Qt::DecorationRole)) {
+
+    if ((index.column() == TorrentContentModelItem::COL_NAME) && (role == Qt::DecorationRole)) {
         if (item->itemType() == TorrentContentModelItem::FolderType)
             return getDirectoryIcon();
         else
-            return getFileIcon();
+            return m_fileIconProvider->icon(QFileInfo(item->name()));
     }
-    if ((index.column() == 0) && (role == Qt::CheckStateRole)) {
+
+    if ((index.column() == TorrentContentModelItem::COL_NAME) && (role == Qt::CheckStateRole)) {
         if (item->data(TorrentContentModelItem::COL_PRIO).toInt() == prio::IGNORED)
             return Qt::Unchecked;
         if (item->data(TorrentContentModelItem::COL_PRIO).toInt() == prio::MIXED)
             return Qt::PartiallyChecked;
         return Qt::Checked;
     }
-    if (role != Qt::DisplayRole)
-        return QVariant();
 
-    return item->data(index.column());
+    if (role == Qt::DisplayRole)
+        return item->data(index.column());
+
+    return QVariant();
 }
 
 Qt::ItemFlags TorrentContentModel::flags(const QModelIndex& index) const
@@ -285,17 +459,18 @@ void TorrentContentModel::clear()
 void TorrentContentModel::setupModelData(const BitTorrent::TorrentInfo &info)
 {
     qDebug("setup model data called");
-    if (info.filesCount() <= 0)
+    const int filesCount = info.filesCount();
+    if (filesCount <= 0)
         return;
 
     emit layoutAboutToBeChanged();
     // Initialize files_index array
-    qDebug("Torrent contains %d files", info.filesCount());
-    m_filesIndex.reserve(info.filesCount());
+    qDebug("Torrent contains %d files", filesCount);
+    m_filesIndex.reserve(filesCount);
 
     TorrentContentModelFolder* currentParent;
     // Iterate over files
-    for (int i = 0; i < info.filesCount(); ++i) {
+    for (int i = 0; i < filesCount; ++i) {
         currentParent = m_rootItem;
         QString path = Utils::Fs::fromNativePath(info.filePath(i));
         // Iterate of parts of the path to create necessary folders
