@@ -30,31 +30,35 @@
 
 #include "transferlistfilterswidget.h"
 
+#include <QCheckBox>
 #include <QDebug>
-#include <QListWidgetItem>
 #include <QIcon>
-#include <QVBoxLayout>
+#include <QListWidgetItem>
 #include <QMenu>
 #include <QMessageBox>
-#include <QCheckBox>
 #include <QScrollArea>
+#include <QVBoxLayout>
 
-#include "transferlistdelegate.h"
-#include "transferlistwidget.h"
-#include "base/preferences.h"
-#include "torrentmodel.h"
-#include "guiiconprovider.h"
-#include "base/utils/fs.h"
-#include "base/utils/string.h"
-#include "autoexpandabledialog.h"
-#include "base/torrentfilter.h"
-#include "base/bittorrent/trackerentry.h"
 #include "base/bittorrent/session.h"
 #include "base/bittorrent/torrenthandle.h"
+#include "base/bittorrent/trackerentry.h"
+#include "base/logger.h"
 #include "base/net/downloadmanager.h"
 #include "base/net/downloadhandler.h"
+#include "base/preferences.h"
+#include "base/torrentfilter.h"
+#include "base/utils/fs.h"
 #include "base/utils/misc.h"
-#include "base/logger.h"
+#include "base/utils/string.h"
+#include "autoexpandabledialog.h"
+#include "categoryfilterwidget.h"
+#include "guiiconprovider.h"
+#include "tagfilterwidget.h"
+#include "torrentmodel.h"
+#include "transferlistdelegate.h"
+#include "transferlistwidget.h"
+
+const QLatin1String GOOGLE_FAVICON_URL("https://www.google.com/s2/favicons?domain=");
 
 FiltersBase::FiltersBase(QWidget *parent, TransferListWidget *transferList)
     : QListWidget(parent)
@@ -74,21 +78,23 @@ FiltersBase::FiltersBase(QWidget *parent, TransferListWidget *transferList)
 #endif
 
     setContextMenuPolicy(Qt::CustomContextMenu);
-    connect(this, SIGNAL(customContextMenuRequested(QPoint)), SLOT(showMenu(QPoint)));
-    connect(this, SIGNAL(currentRowChanged(int)), SLOT(applyFilter(int)));
+    connect(this, &FiltersBase::customContextMenuRequested, this, &FiltersBase::showMenu);
+    connect(this, &FiltersBase::currentRowChanged, this, &FiltersBase::applyFilter);
 
-    connect(BitTorrent::Session::instance(), SIGNAL(torrentAdded(BitTorrent::TorrentHandle *const)), SLOT(handleNewTorrent(BitTorrent::TorrentHandle *const)));
-    connect(BitTorrent::Session::instance(), SIGNAL(torrentAboutToBeRemoved(BitTorrent::TorrentHandle *const)), SLOT(torrentAboutToBeDeleted(BitTorrent::TorrentHandle *const)));
+    connect(BitTorrent::Session::instance(), &BitTorrent::Session::torrentAdded
+            , this, &FiltersBase::handleNewTorrent);
+    connect(BitTorrent::Session::instance(), &BitTorrent::Session::torrentAboutToBeRemoved
+            , this, &FiltersBase::torrentAboutToBeDeleted);
 }
 
 QSize FiltersBase::sizeHint() const
 {
-    QSize size;
-    // Height should be exactly the height of the content
-    size.setHeight(((sizeHintForRow(0) + 2 * spacing()) * (count() + 0.5)));
-    // Width should be exactly the width of the content
-    size.setWidth(sizeHintForColumn(0));
-    return size;
+    return {
+        // Width should be exactly the width of the content
+        sizeHintForColumn(0),
+        // Height should be exactly the height of the content
+        static_cast<int>((sizeHintForRow(0) + 2 * spacing()) * (count() + 0.5)),
+    };
 }
 
 QSize FiltersBase::minimumSizeHint() const
@@ -110,7 +116,8 @@ void FiltersBase::toggleFilter(bool checked)
 StatusFiltersWidget::StatusFiltersWidget(QWidget *parent, TransferListWidget *transferList)
     : FiltersBase(parent, transferList)
 {
-    connect(BitTorrent::Session::instance(), SIGNAL(torrentsUpdated()), SLOT(updateTorrentNumbers()));
+    connect(BitTorrent::Session::instance(), &BitTorrent::Session::torrentsUpdated
+            , this, &StatusFiltersWidget::updateTorrentNumbers);
 
     // Add status filters
     QListWidgetItem *all = new QListWidgetItem(this);
@@ -177,269 +184,10 @@ void StatusFiltersWidget::handleNewTorrent(BitTorrent::TorrentHandle *const) {}
 
 void StatusFiltersWidget::torrentAboutToBeDeleted(BitTorrent::TorrentHandle *const) {}
 
-CategoryFiltersList::CategoryFiltersList(QWidget *parent, TransferListWidget *transferList)
-    : FiltersBase(parent, transferList)
-{
-    connect(BitTorrent::Session::instance(), SIGNAL(torrentCategoryChanged(BitTorrent::TorrentHandle *const, QString)), SLOT(torrentCategoryChanged(BitTorrent::TorrentHandle *const, QString)));
-    connect(BitTorrent::Session::instance(), SIGNAL(categoryAdded(QString)), SLOT(addItem(QString)));
-    connect(BitTorrent::Session::instance(), SIGNAL(categoryRemoved(QString)), SLOT(categoryRemoved(QString)));
-    connect(BitTorrent::Session::instance(), SIGNAL(subcategoriesSupportChanged()), SLOT(subcategoriesSupportChanged()));
-
-    refresh();
-    toggleFilter(Preferences::instance()->getCategoryFilterState());
-}
-
-void CategoryFiltersList::refresh()
-{
-    clear();
-    m_categories.clear();
-    m_totalTorrents = 0;
-    m_totalCategorized = 0;
-
-    QListWidgetItem *allCategories = new QListWidgetItem(this);
-    allCategories->setData(Qt::DisplayRole, QVariant(tr("All (0)", "this is for the category filter")));
-    allCategories->setData(Qt::DecorationRole, GuiIconProvider::instance()->getIcon("inode-directory"));
-    QListWidgetItem *noCategory = new QListWidgetItem(this);
-    noCategory->setData(Qt::DisplayRole, QVariant(tr("Uncategorized (0)")));
-    noCategory->setData(Qt::DecorationRole, GuiIconProvider::instance()->getIcon("inode-directory"));
-
-    foreach (const QString &category, BitTorrent::Session::instance()->categories())
-        addItem(category, false);
-
-    foreach (BitTorrent::TorrentHandle *const torrent, BitTorrent::Session::instance()->torrents())
-        handleNewTorrent(torrent);
-
-    setCurrentRow(0, QItemSelectionModel::SelectCurrent);
-}
-
-void CategoryFiltersList::addItem(const QString &category, bool hasTorrent)
-{
-    if (category.isEmpty()) return;
-
-    int torrentsInCategory = 0;
-    QListWidgetItem *categoryItem = 0;
-
-    bool exists = m_categories.contains(category);
-    if (exists) {
-        torrentsInCategory = m_categories.value(category);
-        categoryItem = item(rowFromCategory(category));
-    }
-    else {
-        categoryItem = new QListWidgetItem();
-        categoryItem->setData(Qt::DecorationRole, GuiIconProvider::instance()->getIcon("inode-directory"));
-    }
-
-    if (hasTorrent)
-        ++torrentsInCategory;
-
-    m_categories.insert(category, torrentsInCategory);
-    categoryItem->setText(tr("%1 (%2)", "category_name (10)").arg(category).arg(torrentsInCategory));
-    if (exists) return;
-
-    Q_ASSERT(count() >= 2);
-    int insPos = count();
-    for (int i = 2; i < count(); ++i) {
-        if (Utils::String::naturalCompareCaseSensitive(category, item(i)->text())) {
-            insPos = i;
-            break;
-        }
-    }
-    QListWidget::insertItem(insPos, categoryItem);
-    updateGeometry();
-}
-
-void CategoryFiltersList::removeItem(const QString &category)
-{
-    if (category.isEmpty()) return;
-
-    int torrentsInCategory = m_categories.value(category) - 1;
-    int row = rowFromCategory(category);
-    if (row < 2) return;
-
-    QListWidgetItem *categoryItem = item(row);
-    categoryItem->setText(tr("%1 (%2)", "category_name (10)").arg(category).arg(torrentsInCategory));
-    m_categories.insert(category, torrentsInCategory);
-}
-
-void CategoryFiltersList::removeSelectedCategory()
-{
-    QList<QListWidgetItem*> items = selectedItems();
-    if (items.size() == 0) return;
-
-    const int categoryRow = row(items.first());
-    if (categoryRow < 2) return;
-
-    BitTorrent::Session::instance()->removeCategory(categoryFromRow(categoryRow));
-    updateGeometry();
-}
-
-void CategoryFiltersList::removeUnusedCategories()
-{
-    foreach (const QString &category, m_categories.keys())
-        if (m_categories[category] == 0)
-            BitTorrent::Session::instance()->removeCategory(category);
-    updateGeometry();
-}
-
-void CategoryFiltersList::torrentCategoryChanged(BitTorrent::TorrentHandle *const torrent, const QString &oldCategory)
-{
-    qDebug() << "Torrent category changed from" << oldCategory << "to" << torrent->category();
-
-    if (torrent->category().isEmpty() && !oldCategory.isEmpty())
-        --m_totalCategorized;
-    else if (!torrent->category().isEmpty() && oldCategory.isEmpty())
-        ++m_totalCategorized;
-
-    item(1)->setText(tr("Uncategorized (%1)").arg(m_totalTorrents - m_totalCategorized));
-
-    if (BitTorrent::Session::instance()->isSubcategoriesEnabled()) {
-        foreach (const QString &subcategory, BitTorrent::Session::expandCategory(oldCategory))
-            removeItem(subcategory);
-        foreach (const QString &subcategory, BitTorrent::Session::expandCategory(torrent->category()))
-            addItem(subcategory, true);
-    }
-    else {
-        removeItem(oldCategory);
-        addItem(torrent->category(), true);
-    }
-}
-
-void CategoryFiltersList::categoryRemoved(const QString &category)
-{
-    m_categories.remove(category);
-    delete takeItem(rowFromCategory(category));
-}
-
-void CategoryFiltersList::subcategoriesSupportChanged()
-{
-    refresh();
-}
-
-void CategoryFiltersList::showMenu(QPoint)
-{
-    QMenu menu(this);
-    QAction *addAct = menu.addAction(GuiIconProvider::instance()->getIcon("list-add"), tr("Add category..."));
-    QAction *removeAct = 0;
-    QAction *removeUnusedAct = 0;
-    if (!selectedItems().empty() && row(selectedItems().first()) > 1)
-        removeAct = menu.addAction(GuiIconProvider::instance()->getIcon("list-remove"), tr("Remove category"));
-    removeUnusedAct = menu.addAction(GuiIconProvider::instance()->getIcon("list-remove"), tr("Remove unused categories"));
-    menu.addSeparator();
-    QAction *startAct = menu.addAction(GuiIconProvider::instance()->getIcon("media-playback-start"), tr("Resume torrents"));
-    QAction *pauseAct = menu.addAction(GuiIconProvider::instance()->getIcon("media-playback-pause"), tr("Pause torrents"));
-    QAction *deleteTorrentsAct = menu.addAction(GuiIconProvider::instance()->getIcon("edit-delete"), tr("Delete torrents"));
-    QAction *act = 0;
-    act = menu.exec(QCursor::pos());
-    if (!act)
-        return;
-
-    if (act == removeAct) {
-        removeSelectedCategory();
-    }
-    else if (act == removeUnusedAct) {
-        removeUnusedCategories();
-    }
-    else if (act == deleteTorrentsAct) {
-        transferList->deleteVisibleTorrents();
-    }
-    else if (act == startAct) {
-        transferList->startVisibleTorrents();
-    }
-    else if (act == pauseAct) {
-        transferList->pauseVisibleTorrents();
-    }
-    else if (act == addAct) {
-        bool ok;
-        QString category = "";
-        bool invalid;
-        do {
-            invalid = false;
-            category = AutoExpandableDialog::getText(this, tr("New Category"), tr("Category:"), QLineEdit::Normal, category, &ok);
-            if (ok && !category.isEmpty()) {
-                if (!BitTorrent::Session::isValidCategoryName(category)) {
-                    QMessageBox::warning(this, tr("Invalid category name"),
-                                         tr("Category name must not contain '\\'.\n"
-                                            "Category name must not start/end with '/'.\n"
-                                            "Category name must not contain '//' sequence."));
-                    invalid = true;
-                }
-                else {
-                    BitTorrent::Session::instance()->addCategory(category);
-                }
-            }
-        } while (invalid);
-    }
-}
-
-void CategoryFiltersList::applyFilter(int row)
-{
-    if (row >= 0)
-        transferList->applyCategoryFilter(categoryFromRow(row));
-}
-
-void CategoryFiltersList::handleNewTorrent(BitTorrent::TorrentHandle *const torrent)
-{
-    Q_ASSERT(torrent);
-
-    ++m_totalTorrents;
-    if (!torrent->category().isEmpty())
-        ++m_totalCategorized;
-
-    item(0)->setText(tr("All (%1)", "this is for the category filter").arg(m_totalTorrents));
-    item(1)->setText(tr("Uncategorized (%1)").arg(m_totalTorrents - m_totalCategorized));
-
-    if (BitTorrent::Session::instance()->isSubcategoriesEnabled()) {
-        foreach (const QString &subcategory, BitTorrent::Session::expandCategory(torrent->category()))
-            addItem(subcategory, true);
-    }
-    else {
-        addItem(torrent->category(), true);
-    }
-}
-
-void CategoryFiltersList::torrentAboutToBeDeleted(BitTorrent::TorrentHandle *const torrent)
-{
-    Q_ASSERT(torrent);
-
-    --m_totalTorrents;
-    if (!torrent->category().isEmpty())
-        --m_totalCategorized;
-
-    item(0)->setText(tr("All (%1)", "this is for the category filter").arg(m_totalTorrents));
-    item(1)->setText(tr("Uncategorized (%1)").arg(m_totalTorrents - m_totalCategorized));
-
-    if (BitTorrent::Session::instance()->isSubcategoriesEnabled()) {
-        foreach (const QString &subcategory, BitTorrent::Session::expandCategory(torrent->category()))
-            removeItem(subcategory);
-    }
-    else {
-        removeItem(torrent->category());
-    }
-}
-
-QString CategoryFiltersList::categoryFromRow(int row) const
-{
-    if (row == 0) return QString(); // All
-    if (row == 1) return QLatin1String(""); // Uncategorized
-
-    const QString &category = item(row)->text();
-    QStringList parts = category.split(" ");
-    Q_ASSERT(parts.size() >= 2);
-    parts.removeLast(); // Remove trailing number
-    return parts.join(" ");
-}
-
-int CategoryFiltersList::rowFromCategory(const QString &category) const
-{
-    Q_ASSERT(!category.isEmpty());
-    for (int i = 2; i<count(); ++i)
-        if (category == categoryFromRow(i)) return i;
-    return -1;
-}
-
 TrackerFiltersList::TrackerFiltersList(QWidget *parent, TransferListWidget *transferList)
     : FiltersBase(parent, transferList)
     , m_totalTorrents(0)
+    , m_downloadTrackerFavicon(true)
 {
     QListWidgetItem *allTrackers = new QListWidgetItem(this);
     allTrackers->setData(Qt::DisplayRole, QVariant(tr("All (0)", "this is for the tracker filter")));
@@ -501,7 +249,7 @@ void TrackerFiltersList::addItem(const QString &tracker, const QString &hash)
         return;
     }
 
-    trackerItem->setText(tr("%1 (%2)", "openbittorrent.com (10)").arg(host).arg(tmp.size()));
+    trackerItem->setText(QString("%1 (%2)").arg(host).arg(tmp.size()));
     if (exists) {
         if (currentRow() == rowFromTracker(host))
             applyFilter(currentRow());
@@ -511,7 +259,7 @@ void TrackerFiltersList::addItem(const QString &tracker, const QString &hash)
     Q_ASSERT(count() >= 4);
     int insPos = count();
     for (int i = 4; i < count(); ++i) {
-        if (Utils::String::naturalCompareCaseSensitive(host, item(i)->text())) {
+        if (Utils::String::naturalLessThan<Qt::CaseSensitive>(host, item(i)->text())) {
             insPos = i;
             break;
         }
@@ -523,7 +271,7 @@ void TrackerFiltersList::addItem(const QString &tracker, const QString &hash)
 void TrackerFiltersList::removeItem(const QString &tracker, const QString &hash)
 {
     QString host = getHost(tracker);
-    QListWidgetItem *trackerItem = 0;
+    QListWidgetItem *trackerItem = nullptr;
     QStringList tmp = m_trackers.value(host);
     int row = 0;
 
@@ -531,7 +279,7 @@ void TrackerFiltersList::removeItem(const QString &tracker, const QString &hash)
         return;
     tmp.removeAll(hash);
 
-    if (host != "") {
+    if (!host.isEmpty()) {
         // Remove from 'Error' and 'Warning' view
         trackerSuccess(hash, tracker);
         row = rowFromTracker(host);
@@ -544,7 +292,8 @@ void TrackerFiltersList::removeItem(const QString &tracker, const QString &hash)
             updateGeometry();
             return;
         }
-        trackerItem->setText(tr("%1 (%2)", "openbittorrent.com (10)").arg(host).arg(tmp.size()));
+        if (trackerItem != nullptr)
+            trackerItem->setText(QString("%1 (%2)").arg(host).arg(tmp.size()));
     }
     else {
         row = 1;
@@ -565,6 +314,19 @@ void TrackerFiltersList::changeTrackerless(bool trackerless, const QString &hash
         removeItem("", hash);
 }
 
+void TrackerFiltersList::setDownloadTrackerFavicon(bool value)
+{
+    if (value == m_downloadTrackerFavicon) return;
+    m_downloadTrackerFavicon = value;
+
+    if (m_downloadTrackerFavicon) {
+        foreach (const QString &tracker, m_trackers.keys()) {
+            if (!tracker.isEmpty())
+                downloadFavicon(QString("http://%1/favicon.ico").arg(tracker));
+        }
+    }
+}
+
 void TrackerFiltersList::trackerSuccess(const QString &hash, const QString &tracker)
 {
     QStringList errored = m_errors.value(hash);
@@ -629,14 +391,21 @@ void TrackerFiltersList::trackerWarning(const QString &hash, const QString &trac
 
 void TrackerFiltersList::downloadFavicon(const QString& url)
 {
+    if (!m_downloadTrackerFavicon) return;
     Net::DownloadHandler *h = Net::DownloadManager::instance()->downloadUrl(url, true);
-    connect(h, SIGNAL(downloadFinished(QString, QString)), this, SLOT(handleFavicoDownload(QString, QString)));
-    connect(h, SIGNAL(downloadFailed(QString, QString)), this, SLOT(handleFavicoFailure(QString, QString)));
+    using Func = void (Net::DownloadHandler::*)(const QString &, const QString &);
+    connect(h, static_cast<Func>(&Net::DownloadHandler::downloadFinished), this
+            , &TrackerFiltersList::handleFavicoDownload);
+    connect(h, static_cast<Func>(&Net::DownloadHandler::downloadFailed), this
+            , &TrackerFiltersList::handleFavicoFailure);
 }
 
 void TrackerFiltersList::handleFavicoDownload(const QString& url, const QString& filePath)
 {
-    QString host = getHost(url);
+    QString host = url.startsWith(GOOGLE_FAVICON_URL)
+                            ? url.mid(GOOGLE_FAVICON_URL.size())
+                            : getHost(url);
+
     if (!m_trackers.contains(host)) {
         Utils::Fs::forceRemove(filePath);
         return;
@@ -650,14 +419,8 @@ void TrackerFiltersList::handleFavicoDownload(const QString& url, const QString&
     QList<QSize> sizes = icon.availableSizes();
     bool invalid = (sizes.isEmpty() || icon.pixmap(sizes.first()).isNull());
     if (invalid) {
-        if (url.endsWith(".ico", Qt::CaseInsensitive)) {
-            Logger::instance()->addMessage(tr("Couldn't decode favicon for URL '%1'. Trying to download favicon in PNG format.").arg(url),
-                                           Log::WARNING);
+        if (url.endsWith(".ico", Qt::CaseInsensitive))
             downloadFavicon(url.left(url.size() - 4) + ".png");
-        }
-        else {
-            Logger::instance()->addMessage(tr("Couldn't decode favicon for URL '%1'.").arg(url), Log::WARNING);
-        }
         Utils::Fs::forceRemove(filePath);
     }
     else {
@@ -668,12 +431,13 @@ void TrackerFiltersList::handleFavicoDownload(const QString& url, const QString&
 
 void TrackerFiltersList::handleFavicoFailure(const QString& url, const QString& error)
 {
-    // Don't use getHost() on the url here. Print the full url. The error might relate to
-    // that.
-    Logger::instance()->addMessage(tr("Couldn't download favicon for URL '%1'. Reason: %2").arg(url).arg(error),
-                                   Log::WARNING);
-    if (url.endsWith(".ico", Qt::CaseInsensitive))
+    Q_UNUSED(error)
+    if (url.endsWith(".ico", Qt::CaseInsensitive)) {
         downloadFavicon(url.left(url.size() - 4) + ".png");
+    }
+    else if (!url.startsWith(GOOGLE_FAVICON_URL)) {
+        downloadFavicon(GOOGLE_FAVICON_URL + getHost(url));
+    }
 }
 
 void TrackerFiltersList::showMenu(QPoint)
@@ -780,7 +544,8 @@ QStringList TrackerFiltersList::getHashes(int row)
 
 TransferListFiltersWidget::TransferListFiltersWidget(QWidget *parent, TransferListWidget *transferList)
     : QFrame(parent)
-    , trackerFilters(0)
+    , m_transferList(transferList)
+    , m_trackerFilters(0)
 {
     Preferences* const pref = Preferences::instance();
 
@@ -819,45 +584,82 @@ TransferListFiltersWidget::TransferListFiltersWidget(QWidget *parent, TransferLi
     QCheckBox *categoryLabel = new QCheckBox(tr("Categories"), this);
     categoryLabel->setChecked(pref->getCategoryFilterState());
     categoryLabel->setFont(font);
+    connect(categoryLabel, &QCheckBox::toggled, this
+            , &TransferListFiltersWidget::onCategoryFilterStateChanged);
     frameLayout->addWidget(categoryLabel);
 
-    CategoryFiltersList *categoryFilters = new CategoryFiltersList(this, transferList);
-    frameLayout->addWidget(categoryFilters);
+    m_categoryFilterWidget = new CategoryFilterWidget(this);
+    connect(m_categoryFilterWidget, &CategoryFilterWidget::actionDeleteTorrentsTriggered
+            , transferList, &TransferListWidget::deleteVisibleTorrents);
+    connect(m_categoryFilterWidget, &CategoryFilterWidget::actionPauseTorrentsTriggered
+            , transferList, &TransferListWidget::pauseVisibleTorrents);
+    connect(m_categoryFilterWidget, &CategoryFilterWidget::actionResumeTorrentsTriggered
+            , transferList, &TransferListWidget::startVisibleTorrents);
+    connect(m_categoryFilterWidget, &CategoryFilterWidget::categoryChanged
+            , transferList, &TransferListWidget::applyCategoryFilter);
+    toggleCategoryFilter(pref->getCategoryFilterState());
+    frameLayout->addWidget(m_categoryFilterWidget);
+
+    QCheckBox *tagsLabel = new QCheckBox(tr("Tags"), this);
+    tagsLabel->setChecked(pref->getTagFilterState());
+    tagsLabel->setFont(font);
+    connect(tagsLabel, &QCheckBox::toggled, this, &TransferListFiltersWidget::onTagFilterStateChanged);
+    frameLayout->addWidget(tagsLabel);
+
+    m_tagFilterWidget = new TagFilterWidget(this);
+    connect(m_tagFilterWidget, &TagFilterWidget::actionDeleteTorrentsTriggered
+            , transferList, &TransferListWidget::deleteVisibleTorrents);
+    connect(m_tagFilterWidget, &TagFilterWidget::actionPauseTorrentsTriggered
+            , transferList, &TransferListWidget::pauseVisibleTorrents);
+    connect(m_tagFilterWidget, &TagFilterWidget::actionResumeTorrentsTriggered
+            , transferList, &TransferListWidget::startVisibleTorrents);
+    connect(m_tagFilterWidget, &TagFilterWidget::tagChanged
+            , transferList, &TransferListWidget::applyTagFilter);
+    toggleTagFilter(pref->getTagFilterState());
+    frameLayout->addWidget(m_tagFilterWidget);
 
     QCheckBox *trackerLabel = new QCheckBox(tr("Trackers"), this);
     trackerLabel->setChecked(pref->getTrackerFilterState());
     trackerLabel->setFont(font);
     frameLayout->addWidget(trackerLabel);
 
-    trackerFilters = new TrackerFiltersList(this, transferList);
-    frameLayout->addWidget(trackerFilters);
+    m_trackerFilters = new TrackerFiltersList(this, transferList);
+    frameLayout->addWidget(m_trackerFilters);
 
-    connect(statusLabel, SIGNAL(toggled(bool)), statusFilters, SLOT(toggleFilter(bool)));
-    connect(statusLabel, SIGNAL(toggled(bool)), pref, SLOT(setStatusFilterState(const bool)));
-    connect(categoryLabel, SIGNAL(toggled(bool)), categoryFilters, SLOT(toggleFilter(bool)));
-    connect(categoryLabel, SIGNAL(toggled(bool)), pref, SLOT(setCategoryFilterState(const bool)));
-    connect(trackerLabel, SIGNAL(toggled(bool)), trackerFilters, SLOT(toggleFilter(bool)));
-    connect(trackerLabel, SIGNAL(toggled(bool)), pref, SLOT(setTrackerFilterState(const bool)));
-    connect(this, SIGNAL(trackerSuccess(const QString &, const QString &)), trackerFilters, SLOT(trackerSuccess(const QString &, const QString &)));
-    connect(this, SIGNAL(trackerError(const QString &, const QString &)), trackerFilters, SLOT(trackerError(const QString &, const QString &)));
-    connect(this, SIGNAL(trackerWarning(const QString &, const QString &)), trackerFilters, SLOT(trackerWarning(const QString &, const QString &)));
+    connect(statusLabel, &QCheckBox::toggled, statusFilters, &StatusFiltersWidget::toggleFilter);
+    connect(statusLabel, &QCheckBox::toggled, pref, &Preferences::setStatusFilterState);
+    connect(trackerLabel, &QCheckBox::toggled, m_trackerFilters, &TrackerFiltersList::toggleFilter);
+    connect(trackerLabel, &QCheckBox::toggled, pref, &Preferences::setTrackerFilterState);
+
+    using Func = void (TransferListFiltersWidget::*)(const QString&, const QString&);
+    connect(this, static_cast<Func>(&TransferListFiltersWidget::trackerSuccess)
+            , m_trackerFilters, &TrackerFiltersList::trackerSuccess);
+    connect(this, static_cast<Func>(&TransferListFiltersWidget::trackerError)
+            , m_trackerFilters, &TrackerFiltersList::trackerError);
+    connect(this, static_cast<Func>(&TransferListFiltersWidget::trackerWarning)
+            , m_trackerFilters, &TrackerFiltersList::trackerWarning);
+}
+
+void TransferListFiltersWidget::setDownloadTrackerFavicon(bool value)
+{
+    m_trackerFilters->setDownloadTrackerFavicon(value);
 }
 
 void TransferListFiltersWidget::addTrackers(BitTorrent::TorrentHandle *const torrent, const QList<BitTorrent::TrackerEntry> &trackers)
 {
     foreach (const BitTorrent::TrackerEntry &tracker, trackers)
-        trackerFilters->addItem(tracker.url(), torrent->hash());
+        m_trackerFilters->addItem(tracker.url(), torrent->hash());
 }
 
 void TransferListFiltersWidget::removeTrackers(BitTorrent::TorrentHandle *const torrent, const QList<BitTorrent::TrackerEntry> &trackers)
 {
     foreach (const BitTorrent::TrackerEntry &tracker, trackers)
-        trackerFilters->removeItem(tracker.url(), torrent->hash());
+        m_trackerFilters->removeItem(tracker.url(), torrent->hash());
 }
 
 void TransferListFiltersWidget::changeTrackerless(BitTorrent::TorrentHandle *const torrent, bool trackerless)
 {
-    trackerFilters->changeTrackerless(trackerless, torrent->hash());
+    m_trackerFilters->changeTrackerless(trackerless, torrent->hash());
 }
 
 void TransferListFiltersWidget::trackerSuccess(BitTorrent::TorrentHandle *const torrent, const QString &tracker)
@@ -874,3 +676,27 @@ void TransferListFiltersWidget::trackerError(BitTorrent::TorrentHandle *const to
 {
     emit trackerError(torrent->hash(), tracker);
 }
+
+void TransferListFiltersWidget::onCategoryFilterStateChanged(bool enabled)
+{
+    toggleCategoryFilter(enabled);
+    Preferences::instance()->setCategoryFilterState(enabled);
+}
+
+void TransferListFiltersWidget::toggleCategoryFilter(bool enabled)
+{
+    m_categoryFilterWidget->setVisible(enabled);
+    m_transferList->applyCategoryFilter(enabled ? m_categoryFilterWidget->currentCategory() : QString());
+}
+
+void TransferListFiltersWidget::onTagFilterStateChanged(bool enabled)
+{
+    toggleTagFilter(enabled);
+    Preferences::instance()->setTagFilterState(enabled);
+}
+
+void TransferListFiltersWidget::toggleTagFilter(bool enabled)
+{
+    m_tagFilterWidget->setVisible(enabled);
+    m_transferList->applyTagFilter(enabled ? m_tagFilterWidget->currentTag() : QString());
+}
