@@ -29,12 +29,13 @@
 #ifndef ABSTRACTWEBAPPLICATION_H
 #define ABSTRACTWEBAPPLICATION_H
 
-#include <QObject>
-#include <QMap>
 #include <QHash>
-#include "base/http/types.h"
-#include "base/http/responsebuilder.h"
+#include <QMap>
+#include <QObject>
+
 #include "base/http/irequesthandler.h"
+#include "base/http/responsebuilder.h"
+#include "base/http/types.h"
 
 struct WebSession;
 struct WebSessionData;
@@ -53,10 +54,10 @@ public:
     explicit AbstractWebApplication(QObject *parent = 0);
     virtual ~AbstractWebApplication();
 
-    Http::Response processRequest(const Http::Request &request, const Http::Environment &env);
+    Http::Response processRequest(const Http::Request &request, const Http::Environment &env) final;
 
 protected:
-    virtual void processRequest() = 0;
+    virtual void doProcessRequest() = 0;
 
     bool isBanned() const;
     int failedAttempts() const;
@@ -78,13 +79,15 @@ protected:
     static QString saveTmpFile(const QByteArray &data);
 
     WebSessionData *session();
-    Http::Request request() const { return request_; }
-    Http::Environment env() const { return env_; }
+    const Http::Request &request() const { return request_; }
+    const Http::Environment &env() const { return env_; }
 
 private slots:
     void UnbanTimerEvent();
     void removeInactiveSessions();
 
+    void reloadDomainList();
+
 private:
     // Persistent data
     QMap<QString, WebSession *> sessions_;
@@ -96,13 +99,18 @@ private:
     Http::Request request_;
     Http::Environment env_;
 
+    QStringList domainList;
+
     QString generateSid();
     bool sessionInitialize();
 
+    QStringMap parseCookie(const Http::Request &request) const;
+    bool isCrossSiteRequest(const Http::Request &request) const;
+    bool validateHostHeader(const QStringList &domains) const;
+
     static void translateDocument(QString &data);
 
     static const QStringMap CONTENT_TYPE_BY_EXT;
-    static QStringMap initializeContentTypeByExtMap();
 };
 
 #endif // ABSTRACTWEBAPPLICATION_H
