@@ -32,12 +32,12 @@
 #include <QStringList>
 #include "abstractwebapplication.h"
 
-class WebApplication: public AbstractWebApplication
+class WebApplication : public AbstractWebApplication
 {
     Q_DISABLE_COPY(WebApplication)
 
 public:
-    explicit WebApplication(QObject* parent = 0);
+    explicit WebApplication(QObject *parent = nullptr);
 
 private:
     // Actions
@@ -56,6 +56,8 @@ private:
     void action_query_propertiesFiles();
     void action_query_getLog();
     void action_query_getPeerLog();
+    void action_query_getPieceHashes();
+    void action_query_getPieceStates();
     void action_sync_maindata();
     void action_sync_torrent_peers();
     void action_command_shutdown();
@@ -88,6 +90,9 @@ private:
     void action_command_decreasePrio();
     void action_command_topPrio();
     void action_command_bottomPrio();
+    void action_command_setLocation();
+    void action_command_rename();
+    void action_command_setAutoTMM();
     void action_command_recheck();
     void action_command_setCategory();
     void action_command_addCategory();
@@ -103,7 +108,7 @@ private:
     QString action_;
     QStringList args_;
 
-    void processRequest();
+    void doProcessRequest() override;
 
     bool isPublicScope();
     void parsePath();
