@@ -19,7 +19,7 @@ if (typeof localStorage == 'undefined') {
                 duration: 365 * 10
             });
         }
-    }
+    };
 }
 
 function getLocalStorageItem(name, defaultVal) {
@@ -47,7 +47,7 @@ initializeWindows = function() {
         new Event(e).stop();
         new MochaUI.Window({
             id: 'downloadPage',
-            title: "QBT_TR(Download from URLs)QBT_TR",
+            title: "QBT_TR(Download from URLs)QBT_TR[CONTEXT=downloadFromURL]",
             loadMethod: 'iframe',
             contentURL: 'download.html',
             scrollbars: true,
@@ -57,7 +57,7 @@ initializeWindows = function() {
             paddingVertical: 0,
             paddingHorizontal: 0,
             width: 500,
-            height: 360
+            height: 420
         });
         updateMainData();
     });
@@ -66,7 +66,7 @@ initializeWindows = function() {
         new Event(e).stop();
         new MochaUI.Window({
             id: 'preferencesPage',
-            title: "QBT_TR(Options)QBT_TR",
+            title: "QBT_TR(Options)QBT_TR[CONTEXT=OptionsDialog]",
             loadMethod: 'xhr',
             toolbar: true,
             contentURL: 'preferences_content.html',
@@ -88,7 +88,7 @@ initializeWindows = function() {
         new Event(e).stop();
         new MochaUI.Window({
             id: 'uploadPage',
-            title: "QBT_TR(Upload local torrent)QBT_TR",
+            title: "QBT_TR(Upload local torrent)QBT_TR[CONTEXT=HttpServer]",
             loadMethod: 'iframe',
             contentURL: 'upload.html',
             scrollbars: true,
@@ -97,7 +97,7 @@ initializeWindows = function() {
             paddingVertical: 0,
             paddingHorizontal: 0,
             width: 500,
-            height: 200
+            height: 260
         });
         updateMainData();
     });
@@ -105,7 +105,7 @@ initializeWindows = function() {
     globalUploadLimitFN = function() {
         new MochaUI.Window({
             id: 'uploadLimitPage',
-            title: "QBT_TR(Global Upload Speed Limit)QBT_TR",
+            title: "QBT_TR(Global Upload Speed Limit)QBT_TR[CONTEXT=MainWindow]",
             loadMethod: 'iframe',
             contentURL: 'uploadlimit.html?hashes=global',
             scrollbars: false,
@@ -116,17 +116,17 @@ initializeWindows = function() {
             width: 424,
             height: 80
         });
-    }
+    };
 
     uploadLimitFN = function() {
-        var h = torrentsTable.selectedRowsIds();
-        if (h.length) {
-            var hash = h[0];
+        var hashes = torrentsTable.selectedRowsIds();
+        if (hashes.length) {
+            var hash = hashes[0];
             new MochaUI.Window({
                 id: 'uploadLimitPage',
-                title: "QBT_TR(Torrent Upload Speed Limiting)QBT_TR",
+                title: "QBT_TR(Torrent Upload Speed Limiting)QBT_TR[CONTEXT=TransferListWidget]",
                 loadMethod: 'iframe',
-                contentURL: 'uploadlimit.html?hashes=' + h.join("|"),
+                contentURL: 'uploadlimit.html?hashes=' + hashes.join("|"),
                 scrollbars: false,
                 resizable: false,
                 maximizable: false,
@@ -139,13 +139,13 @@ initializeWindows = function() {
     };
 
     toggleSequentialDownloadFN = function() {
-        var h = torrentsTable.selectedRowsIds();
-        if (h.length) {
+        var hashes = torrentsTable.selectedRowsIds();
+        if (hashes.length) {
             new Request({
                 url: 'command/toggleSequentialDownload',
                 method: 'post',
                 data: {
-                    hashes: h.join("|")
+                    hashes: hashes.join("|")
                 }
             }).send();
             updateMainData();
@@ -153,13 +153,13 @@ initializeWindows = function() {
     };
 
     toggleFirstLastPiecePrioFN = function() {
-        var h = torrentsTable.selectedRowsIds();
-        if (h.length) {
+        var hashes = torrentsTable.selectedRowsIds();
+        if (hashes.length) {
             new Request({
                 url: 'command/toggleFirstLastPiecePrio',
                 method: 'post',
                 data: {
-                    hashes: h.join("|")
+                    hashes: hashes.join("|")
                 }
             }).send();
             updateMainData();
@@ -167,14 +167,14 @@ initializeWindows = function() {
     };
 
     setSuperSeedingFN = function(val) {
-        var h = torrentsTable.selectedRowsIds();
-        if (h.length) {
+        var hashes = torrentsTable.selectedRowsIds();
+        if (hashes.length) {
             new Request({
                 url: 'command/setSuperSeeding',
                 method: 'post',
                 data: {
                     value: val,
-                    hashes: h.join("|")
+                    hashes: hashes.join("|")
                 }
             }).send();
             updateMainData();
@@ -182,14 +182,14 @@ initializeWindows = function() {
     };
 
     setForceStartFN = function() {
-        var h = torrentsTable.selectedRowsIds();
-        if (h.length) {
+        var hashes = torrentsTable.selectedRowsIds();
+        if (hashes.length) {
             new Request({
                 url: 'command/setForceStart',
                 method: 'post',
                 data: {
                     value: 'true',
-                    hashes: h.join("|")
+                    hashes: hashes.join("|")
                 }
             }).send();
             updateMainData();
@@ -199,7 +199,7 @@ initializeWindows = function() {
     globalDownloadLimitFN = function() {
         new MochaUI.Window({
             id: 'downloadLimitPage',
-            title: "QBT_TR(Global Download Speed Limit)QBT_TR",
+            title: "QBT_TR(Global Download Speed Limit)QBT_TR[CONTEXT=MainWindow]",
             loadMethod: 'iframe',
             contentURL: 'downloadlimit.html?hashes=global',
             scrollbars: false,
@@ -210,17 +210,32 @@ initializeWindows = function() {
             width: 424,
             height: 80
         });
-    }
+    };
+
+    StatisticsLinkFN = function() {
+        new MochaUI.Window({
+            id: 'statisticspage',
+            title: 'QBT_TR(Statistics)QBT_TR[CONTEXT=StatsDialog]',
+            loadMethod: 'xhr',
+            contentURL: 'statistics.html',
+            scrollbars: false,
+            resizable: false,
+            maximizable: false,
+            width: 275,
+            height: 370,
+            padding: 10
+        });
+    };
 
     downloadLimitFN = function() {
-        var h = torrentsTable.selectedRowsIds();
-        if (h.length) {
-            var hash = h[0];
+        var hashes = torrentsTable.selectedRowsIds();
+        if (hashes.length) {
+            var hash = hashes[0];
             new MochaUI.Window({
                 id: 'downloadLimitPage',
-                title: "QBT_TR(Torrent Download Speed Limiting)QBT_TR",
+                title: "QBT_TR(Torrent Download Speed Limiting)QBT_TR[CONTEXT=TransferListWidget]",
                 loadMethod: 'iframe',
-                contentURL: 'downloadlimit.html?hashes=' + h.join("|"),
+                contentURL: 'downloadlimit.html?hashes=' + hashes.join("|"),
                 scrollbars: false,
                 resizable: false,
                 maximizable: false,
@@ -233,13 +248,13 @@ initializeWindows = function() {
     };
 
     deleteFN = function() {
-        var h = torrentsTable.selectedRowsIds();
-        if (h.length) {
+        var hashes = torrentsTable.selectedRowsIds();
+        if (hashes.length) {
             new MochaUI.Window({
                 id: 'confirmDeletionPage',
-                title: "QBT_TR(Deletion confirmation)QBT_TR",
+                title: "QBT_TR(Deletion confirmation)QBT_TR[CONTEXT=confirmDeletionDlg]",
                 loadMethod: 'iframe',
-                contentURL: 'confirmdeletion.html?hashes=' + h.join("|"),
+                contentURL: 'confirmdeletion.html?hashes=' + hashes.join("|"),
                 scrollbars: false,
                 resizable: false,
                 maximizable: false,
@@ -257,9 +272,9 @@ initializeWindows = function() {
     });
 
     pauseFN = function() {
-        var h = torrentsTable.selectedRowsIds();
-        if (h.length) {
-            h.each(function(hash, index) {
+        var hashes = torrentsTable.selectedRowsIds();
+        if (hashes.length) {
+            hashes.each(function(hash, index) {
                 new Request({
                     url: 'command/pause',
                     method: 'post',
@@ -273,9 +288,9 @@ initializeWindows = function() {
     };
 
     startFN = function() {
-        var h = torrentsTable.selectedRowsIds();
-        if (h.length) {
-            h.each(function(hash, index) {
+        var hashes = torrentsTable.selectedRowsIds();
+        if (hashes.length) {
+            hashes.each(function(hash, index) {
                 new Request({
                     url: 'command/resume',
                     method: 'post',
@@ -288,10 +303,31 @@ initializeWindows = function() {
         }
     };
 
+    autoTorrentManagementFN = function() {
+        var hashes = torrentsTable.selectedRowsIds();
+        if (hashes.length) {
+            var enable = false;
+            hashes.each(function(hash, index) {
+                var row = torrentsTable.rows[hash];
+                if (!row.full_data.auto_tmm)
+                    enable = true;
+            });
+            new Request({
+                url: 'command/setAutoTMM',
+                method: 'post',
+                data: {
+                    hashes: hashes.join("|"),
+                    enable: enable
+                }
+            }).send();
+            updateMainData();
+        }
+    };
+
     recheckFN = function() {
-        var h = torrentsTable.selectedRowsIds();
-        if (h.length) {
-            h.each(function(hash, index) {
+        var hashes = torrentsTable.selectedRowsIds();
+        if (hashes.length) {
+            hashes.each(function(hash, index) {
                 new Request({
                     url: 'command/recheck',
                     method: 'post',
@@ -304,14 +340,57 @@ initializeWindows = function() {
         }
     };
 
+    setLocationFN = function() {
+        var hashes = torrentsTable.selectedRowsIds();
+        if (hashes.length) {
+            new MochaUI.Window({
+                id: 'setLocationPage',
+                title: "QBT_TR(Set location)QBT_TR[CONTEXT=TransferListWidget]",
+                loadMethod: 'iframe',
+                contentURL: 'setlocation.html?hashes=' + hashes.join('|'),
+                scrollbars: false,
+                resizable: false,
+                maximizable: false,
+                paddingVertical: 0,
+                paddingHorizontal: 0,
+                width: 250,
+                height: 100
+            });
+        }
+    };
+
+    renameFN = function() {
+        var hashes = torrentsTable.selectedRowsIds();
+        if (hashes.length == 1) {
+            var hash = hashes[0];
+            var row = torrentsTable.rows[hash];
+            if (row) {
+                var name = row.full_data.name;
+                new MochaUI.Window({
+                    id: 'renamePage',
+                    title: "QBT_TR(Rename)QBT_TR[CONTEXT=TransferListWidget]",
+                    loadMethod: 'iframe',
+                    contentURL: 'rename.html?hash=' + hashes[0] + '&name=' + name,
+                    scrollbars: false,
+                    resizable: false,
+                    maximizable: false,
+                    paddingVertical: 0,
+                    paddingHorizontal: 0,
+                    width: 250,
+                    height: 100
+                });
+            }
+        }
+    };
+
     torrentNewCategoryFN = function () {
-        var h = torrentsTable.selectedRowsIds();
-        if (h.length) {
+        var hashes = torrentsTable.selectedRowsIds();
+        if (hashes.length) {
             new MochaUI.Window({
                 id: 'newCategoryPage',
-                title: "QBT_TR(New Category)QBT_TR",
+                title: "QBT_TR(New Category)QBT_TR[CONTEXT=TransferListWidget]",
                 loadMethod: 'iframe',
-                contentURL: 'newcategory.html?hashes=' + h.join('|'),
+                contentURL: 'newcategory.html?hashes=' + hashes.join('|'),
                 scrollbars: false,
                 resizable: false,
                 maximizable: false,
@@ -326,14 +405,14 @@ initializeWindows = function() {
     torrentSetCategoryFN = function (categoryHash) {
         var categoryName = '';
         if (categoryHash != 0)
-            var categoryName = category_list[categoryHash].name;
-        var h = torrentsTable.selectedRowsIds();
-        if (h.length) {
+            categoryName = category_list[categoryHash].name;
+        var hashes = torrentsTable.selectedRowsIds();
+        if (hashes.length) {
             new Request({
                 url: 'command/setCategory',
                 method: 'post',
                 data: {
-                    hashes: h.join("|"),
+                    hashes: hashes.join("|"),
                     category: categoryName
                 }
             }).send();
@@ -343,7 +422,7 @@ initializeWindows = function() {
     createCategoryFN = function () {
         new MochaUI.Window({
             id: 'newCategoryPage',
-            title: "QBT_TR(New Category)QBT_TR",
+            title: "QBT_TR(New Category)QBT_TR[CONTEXT=CategoryFilterWidget]",
             loadMethod: 'iframe',
             contentURL: 'newcategory.html',
             scrollbars: false,
@@ -372,7 +451,7 @@ initializeWindows = function() {
     deleteUnusedCategoriesFN = function () {
         var categories = [];
         for (var hash in category_list) {
-            if (torrentsTable.getFilteredTorrentsNumber('all', hash) == 0)
+            if (torrentsTable.getFilteredTorrentsNumber('all', hash) === 0)
                 categories.push(category_list[hash].name);
         }
         new Request({
@@ -386,9 +465,9 @@ initializeWindows = function() {
     };
 
     startTorrentsByCategoryFN = function (categoryHash) {
-        var h = torrentsTable.getFilteredTorrentsHashes('all', categoryHash);
-        if (h.length) {
-            h.each(function (hash, index) {
+        var hashes = torrentsTable.getFilteredTorrentsHashes('all', categoryHash);
+        if (hashes.length) {
+            hashes.each(function (hash, index) {
                 new Request({
                     url: 'command/resume',
                     method: 'post',
@@ -402,9 +481,9 @@ initializeWindows = function() {
     };
 
     pauseTorrentsByCategoryFN = function (categoryHash) {
-        var h = torrentsTable.getFilteredTorrentsHashes('all', categoryHash);
-        if (h.length) {
-            h.each(function (hash, index) {
+        var hashes = torrentsTable.getFilteredTorrentsHashes('all', categoryHash);
+        if (hashes.length) {
+            hashes.each(function (hash, index) {
                 new Request({
                     url: 'command/pause',
                     method: 'post',
@@ -418,13 +497,13 @@ initializeWindows = function() {
     };
 
     deleteTorrentsByCategoryFN = function (categoryHash) {
-        var h = torrentsTable.getFilteredTorrentsHashes('all', categoryHash);
-        if (h.length) {
+        var hashes = torrentsTable.getFilteredTorrentsHashes('all', categoryHash);
+        if (hashes.length) {
             new MochaUI.Window({
                 id: 'confirmDeletionPage',
-                title: "QBT_TR(Deletion confirmation)QBT_TR",
+                title: "QBT_TR(Deletion confirmation)QBT_TR[CONTEXT=confirmDeletionDlg]",
                 loadMethod: 'iframe',
-                contentURL: 'confirmdeletion.html?hashes=' + h.join("|"),
+                contentURL: 'confirmdeletion.html?hashes=' + hashes.join("|"),
                 scrollbars: false,
                 resizable: false,
                 maximizable: false,
@@ -436,6 +515,36 @@ initializeWindows = function() {
         }
     };
 
+    copyNameFN = function() {
+        var selectedRows = torrentsTable.selectedRowsIds();
+        var names = [];
+        if (selectedRows.length) {
+            var rows = torrentsTable.getFilteredAndSortedRows();
+            for (var i = 0; i < selectedRows.length; i++) {
+                var hash = selectedRows[i];
+                names.push(rows[hash].full_data.name);
+            }
+        }
+        return names.join("\n");
+    };
+
+    copyMagnetLinkFN = function() {
+        var selectedRows = torrentsTable.selectedRowsIds();
+        var magnets = [];
+        if (selectedRows.length) {
+            var rows = torrentsTable.getFilteredAndSortedRows();
+            for (var i = 0; i < selectedRows.length; i++) {
+                var hash = selectedRows[i];
+                magnets.push(rows[hash].full_data.magnet_uri);
+            }
+        }
+        return magnets.join("\n");
+    };
+
+    copyHashFN = function() {
+        return torrentsTable.selectedRowsIds().join("\n");
+    };
+
     ['pauseAll', 'resumeAll'].each(function(item) {
         addClickEvent(item, function(e) {
             new Event(e).stop();
@@ -449,9 +558,9 @@ initializeWindows = function() {
     ['pause', 'resume', 'recheck'].each(function(item) {
         addClickEvent(item, function(e) {
             new Event(e).stop();
-            var h = torrentsTable.selectedRowsIds();
-            if (h.length) {
-                h.each(function(hash, index) {
+            var hashes = torrentsTable.selectedRowsIds();
+            if (hashes.length) {
+                hashes.each(function(hash, index) {
                     new Request({
                         url: 'command/' + item,
                         method: 'post',
@@ -473,24 +582,24 @@ initializeWindows = function() {
     });
 
     setPriorityFN = function(cmd) {
-        var h = torrentsTable.selectedRowsIds();
-        if (h.length) {
+        var hashes = torrentsTable.selectedRowsIds();
+        if (hashes.length) {
             new Request({
                 url: 'command/' + cmd,
                 method: 'post',
                 data: {
-                    hashes: h.join("|")
+                    hashes: hashes.join("|")
                 }
             }).send();
             updateMainData();
         }
-    }
+    };
 
     addClickEvent('about', function(e) {
         new Event(e).stop();
         new MochaUI.Window({
             id: 'aboutpage',
-            title: 'QBT_TR(About)QBT_TR',
+            title: 'QBT_TR(About)QBT_TR[CONTEXT=AboutDlg]',
             loadMethod: 'xhr',
             contentURL: 'about.html',
             width: 550,
@@ -503,7 +612,7 @@ initializeWindows = function() {
         new Event(e).stop();
         new Request({
             url: 'logout',
-            method: 'get',
+            method: 'post',
             onSuccess: function() {
                 window.location.reload();
             }
@@ -512,11 +621,11 @@ initializeWindows = function() {
 
     addClickEvent('shutdown', function(e) {
         new Event(e).stop();
-        if (confirm('QBT_TR(Are you sure you want to quit qBittorrent?)QBT_TR')) {
+        if (confirm('QBT_TR(Are you sure you want to quit qBittorrent?)QBT_TR[CONTEXT=MainWindow]')) {
             new Request({
                 url: 'command/shutdown',
                 onSuccess: function() {
-                    document.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?><!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\"><html xmlns=\"http://www.w3.org/1999/xhtml\"><head><title>QBT_TR(qBittorrent has been shutdown.)QBT_TR</title><style type=\"text/css\">body { text-align: center; }</style></head><body><h1>QBT_TR(qBittorrent has been shutdown.)QBT_TR</h1></body></html>");
+                    document.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?><!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\"><html xmlns=\"http://www.w3.org/1999/xhtml\"><head><title>QBT_TR(qBittorrent has been shutdown.)QBT_TR[CONTEXT=HttpServer]</title><style type=\"text/css\">body { text-align: center; }</style></head><body><h1>QBT_TR(qBittorrent has been shutdown.)QBT_TR[CONTEXT=HttpServer]</h1></body></html>");
                     stop();
                 }
             }).send();
@@ -529,4 +638,4 @@ initializeWindows = function() {
             new Event(e).stop();
         });
     });
-}
+};
